#include "Adaptivity.h"

Adaptivity::Adaptivity(){

  refinement_strategy = fixed_fraction;
  refine_fraction = 0.1;
  
  no_adaptive_iterations = 3;
}


int Adaptivity::SolveTransientProblemAdaptive( TranProblem* primal, TranProblem* dual ){

  MV_Vector<real> contResidual;
  int adaptive_it;

  MV_Vector<Grid>* GRID;
  //int fine_grid = GRID->size()-1;
 
  int initial_comp_grid;
  globals->Get("current computational level",&initial_comp_grid);

  MV_ColMat<real> uk;

  for ( adaptive_it=0; adaptive_it < no_adaptive_iterations; adaptive_it++ ){
    
    cout << "Adaptive loop " << adaptive_it+1 << endl;
    
    globals->Set("current computational level", adaptive_it+initial_comp_grid);
    globals->Set("current output level", adaptive_it+initial_comp_grid);
    globals->Set("current input level primal", adaptive_it+initial_comp_grid);

    primal->Solve();
    //dual->Solve();
      
    cout << "check1" << endl;
    primal->GetContResidual(contResidual);
    cout << "check2" << endl;
    GRID = primal->GetGridVector(); 
    cout << "check3" << endl;
    if ( adaptive_it < no_adaptive_iterations-1 ) SpaceMeshRefinement( GRID, &contResidual );
    cout << "check4" << endl;
  }
 
  return 0;	
}


int Adaptivity::SpaceMeshRefinement( MV_Vector<Grid>* GRID, MV_Vector<real>* errMeasure ){
  
  int no_grids = GRID->size();
  int no_elms  = (*GRID)(no_grids-1).GetNoCells();
  
  int i,j,jmin;
  
  int ref_no_elms;
  real err_min_val;
  MV_Vector<int> ref_elms_idx;
  MV_Vector<real> ref_elms_val;

  //errMeasure->newsize(no_elms);
  //for ( i=0; i < no_elms; i++ ) (*errMeasure)(i) = sin(1.0*i);

  switch( refinement_strategy ){ 
  case fixed_fraction: 

    ref_no_elms = int(floor(refine_fraction*no_elms));
    ref_elms_idx.newsize(ref_no_elms);
    ref_elms_val.newsize(ref_no_elms);
    ref_elms_idx = 0;
    ref_elms_val = 0.0;
  
  cout << "check 1" << endl;
    err_min_val = 0.0;
    for ( i=0; i < no_elms; i++ ){
      if ( (*errMeasure)(i) > err_min_val ){
	err_min_val = 1000000.0;
	for ( j=0; j < ref_no_elms; j++ ){
	  if ( ref_elms_val(j) < err_min_val ){
	    err_min_val = ref_elms_val(j);
	    jmin = j;	  }	      
	}
	ref_elms_idx(jmin) = i;
	ref_elms_val(jmin) = (*errMeasure)(i);
      }
    }
  cout << "check 2" << endl;

    break;
  default:
    cout << "ERROR: Wrong adaptive refinement strategy" << endl;
    exit(1);
  }
  
  MV_Vector<int> elMarker(no_elms);
  elMarker = 0;
  for ( i=0; i < ref_no_elms; i++ ) elMarker(ref_elms_idx(i)) = 1;

  /*  
  for ( i=0; i < ref_no_elms; i++ ) cout << "ref.coord: (" << (*GRID)(no_grids-1).getCoor((*GRID)(no_grids-1).loc2glob(ref_elms_idx(i),0),0)
					 << "," << (*GRID)(no_grids-1).getCoor((*GRID)(no_grids-1).loc2glob(ref_elms_idx(i),0),1)  << "," 
                                         << (*GRID)(no_grids-1).getCoor((*GRID)(no_grids-1).loc2glob(ref_elms_idx(i),0),2) <<")" << endl;
  */

  MV_Vector<Grid> tmpGRID(no_grids+1);
  for ( i=0; i < no_grids; i++ ){
    cout << "check" << endl;
    (*GRID)(i).Display();
    cout << "check" << endl;
    tmpGRID(i) = (*GRID)(i);
    //cout << tmpGRID(i).ok() << endl;
    //cout << (*GRID)(i).ok() << endl;
    cout << "check" << endl;
    tmpGRID(i).Display();
    cout << "check" << endl;
  }
  (*GRID).newsize(no_grids+1);
  for ( i=0; i < no_grids; i++ ){
    (*GRID)(i) = tmpGRID(i);
    (*GRID)(i).Display();
  }

  tmpGRID(no_grids-1).refine(&tmpGRID(no_grids),elMarker);
  //tmpGRID(no_grids).getSurfaceTriangulation().initRefinedGrid(tmpGRID(no_grids),false);
  tmpGRID(no_grids-1).Display();
  tmpGRID(no_grids).Display();
  tmpGRID(no_grids).presentSurfGridAndGeometry();
  
  (*GRID)(no_grids) = tmpGRID(i);
  (*GRID)(no_grids).Display();
  
  return 0;  
  
}


