// Original author: Johan Hoffman. Added to Kraftwerk 2002-01-14.
// Copyright (C) 2002 Johan Hoffman and Anders Logg.
// Licensed under the GNU GPL Version 2.

#ifndef __GLOBALFIELD_HH
#define __GLOBALFIELD_HH

#include <kw_constants.h>

class Grid;
class Vector;

enum representation { NONE, NODAL, CELL, CONSTANT };

enum nodalbasis { NO_BASIS, LINE_BASIS, TRI_BASIS, QUAD_BASIS, TET_BASIS, HEX_BASIS, PYR_BASIS, 
		  PRISM_BASIS, SIMPLEX_BASIS, HYPERCUBE_BASIS };
 
class GlobalField{
  
public:

  GlobalField();
  GlobalField( Grid* g );

  ~GlobalField();

  void AttachGrid( Grid* g );
  
  int GetNoCells();
  int GetNoNodes();
  int GetVectorDim(int cell);

  void InitByNodes(Vector *nodal_values, int nvd);
  void InitByCells(Vector *cell_values, int nvd);
  void InitByConstant(real c);
  
  int GetNodalBasis(int cell);

  void GetLocalNodes(int cell, int component, real *local_nodes);
  void GetLocalNodes(int cell, real *local_nodes);

  representation GetRep() { return rep; }  

  real GetCellValue(int cell, int component);
  real GetCellValue(int cell);  
  real GetNodalValue(int nod, int component);
  real GetNodalValue(int nod);
  real GetConstantValue(); 

 private:

  Grid* grid;
  bool grid_init;

  int no_nodes;
  int no_cells;
  int nsd;
  int nvd;

  int no_shape_functions;

  representation rep;
  nodalbasis locBasis;

  bool uniform_nodal_basis;
  
  Vector *nodal_values;
  Vector *cell_values;
  real constant;    

};

#endif





