// Original author: Johan Hoffman. Added to Kraftwerk 2002-01-14.
// Copyright (C) 2002 Johan Hoffman and Anders Logg.
// Licensed under the GNU GPL Version 2.

#include "GridRelation.h"

GridRelation::GridRelation()
{
  prol_init    = false;
  res_init     = false;
  el_prol_init = false;
  el_res_init  = false;
} 

GridRelation::GridRelation( Grid* g1, Grid* g2 ) : grid1(g1),grid2(g2)
{
  prol_init    = false;
  res_init     = false;
  el_prol_init = false;
  el_res_init  = false;

  allElmsOfSameType = 1;

  int i,j,k;
  int noNodesInElm = 0;

  noNodes1 = grid1->GetNoNodes();
  noNodes2 = grid2->GetNoNodes();
  noElms1 = grid1->GetNoCells(); 
  noElms2 = grid2->GetNoCells(); 

  settings->Get("dimension", &nsd);
  
  if ( noNodes1 < noNodes2 ){
    coarseNoNodes = noNodes1;
    fineNoNodes = noNodes2;
    coarseNoElms = noElms1;
    fineNoElms = noElms2;
    finegrid = 2;
    coarsegrid = 1;
  }
  else {
    coarseNoNodes = noNodes2;
    fineNoNodes = noNodes1;
    coarseNoElms = noElms2;
    fineNoElms = noElms1;
    finegrid = 1;
    coarsegrid = 2;
  }

  baryCoord.newsize(20); // Size (=20) should be larger than max(noNodesInElm) 
} 


void GridRelation:: operator = (GridRelation& gr)
{
  prol_init    = false;
  res_init     = false;
  el_prol_init = false;
  el_res_init  = false;

  this->grid1 = gr.GetGrid1();
  this->grid2 = gr.GetGrid2();

  allElmsOfSameType = 1;

  int i,j,k;
  int noNodesInElm = 0;

  noNodes1 = grid1->GetNoNodes();
  noNodes2 = grid2->GetNoNodes();
  noElms1 = grid1->GetNoCells(); 
  noElms2 = grid2->GetNoCells(); 

  settings->Get("dimension", &nsd);
  
  if ( noNodes1 < noNodes2 ){
    coarseNoNodes = noNodes1;
    fineNoNodes = noNodes2;
    coarseNoElms = noElms1;
    fineNoElms = noElms2;
    finegrid = 2;
    coarsegrid = 1;
  }
  else {
    coarseNoNodes = noNodes2;
    fineNoNodes = noNodes1;
    coarseNoElms = noElms2;
    fineNoElms = noElms1;
    finegrid = 1;
    coarsegrid = 2;
  }
  
  baryCoord.newsize(20); // Size (=20) should be larger than max(noNodesInElm) 

} 


bool GridRelation::NodalProlongationInitialized(){

  return prol_init;
}


bool GridRelation::NodalRestrictionInitialized(){

  return res_init;
}


bool GridRelation::ElementProlongationInitialized(){

  return el_prol_init;
}


bool GridRelation::ElementRestrictionInitialized(){

  return el_res_init;
}



Grid* GridRelation::GetCoarseGrid(){
  
  if ( coarsegrid == 1 ) return grid1;
  if ( coarsegrid == 2 ) return grid2;

}

Grid* GridRelation::GetFineGrid(){
  
  if ( finegrid == 1 ) return grid1;
  if ( finegrid == 2 ) return grid2;

}

int GridRelation::GetNoNodes1(){
  return noNodes1;
}     

int GridRelation::GetNoNodes2(){
  return noNodes2;
}     

int GridRelation::GetNoCells1(){
  return noElms1;
}     

int GridRelation::GetNoCells2(){
  return noElms2;
}     

int GridRelation::GetNoSpaceDim(){
  return nsd;
}     

int GridRelation::GetFineNoNodes(){
  return fineNoNodes;
}     

int GridRelation::GetFineNoElms(){
  return fineNoElms;
}     

int GridRelation::GetCoarseNoNodes(){
  return coarseNoNodes;
}     

int GridRelation::GetCoarseNoElms(){
  return coarseNoElms;
}     



int GridRelation::writeProlongationMatrix( char *file )
{
  /*
  FILE *fp;

  fp = fopen(file,"w");

  fprintf(fp, "%i %i %i\n", fineNoNodes, coarseNoNodes, maxNoNodesInElm * fineNoNodes );

  int i;
  for( i = 0; i < fineNoNodes + 1; i++){
    fprintf(fp, "%i\n", prolMat.irow(i));
  }
  for( i = 0; i < maxNoNodesInElm * fineNoNodes; i++){
    fprintf(fp, "%i\n", prolMat.jcol(i));
  }
  for( i = 0; i < maxNoNodesInElm * fineNoNodes; i++){
    fprintf(fp, "%i\n", prolCoeff(i));
  }
  
  fclose(fp);

  printf("Prolongation matrix written to file: %s\n", file);
  */
  return 0;
}



int GridRelation::initNodalProlongation(){

  int prolstatus;

  cout << "Initializing nodal prolongation mapping..." << endl;
  
  prolstatus = prolongation(); // Create prolongation matrix

  if ( prolstatus == -1 ){
    cout << "ERROR: In initialising Prolongation matrix!!" << endl;
    exit(1);
  }
  cout << "finished." << endl;
  
  prol_init = true;

  return 0;
}



int GridRelation::initNodalRestriction(){

  int resstatus;

  cout << "Initializing nodal restriction mapping..." << endl;
  
  resstatus = restriction();   // Create restriction matrix

  if ( resstatus == -1 ){
    cout << "ERROR: In initialising Restriction matrix!!" << endl;
    exit(1);
  }
  cout << "finished." << endl;

  res_init = true;

  return 0;
}


int GridRelation::initNodalMap(){

  int prolstatus,resstatus;

  cout << "Initializing nodal mapping...creating prolongation matrix..." ;
  prolstatus = prolongation();                      // Create prolongation matrix
  cout << "creating restriction matrix..." ;
  resstatus = restriction();                        // Create restriction matrix
  
  if ( prolstatus == -1 ){
    cout << "ERROR: In initialising Prolongation matrix!!" << endl;
    exit(1);
  }
  if ( resstatus == -1 ){
    cout << "ERROR: In initialising Restriction matrix!!" << endl;
    exit(1);
  }
  cout << "finished." << endl;

  prol_init = true;
  res_init = true;

  return 0;
}


int GridRelation::initNestElMap(){

  int prolstatus,resstatus;

  cout << "Initializing nested element mapping...creating prolongation matrix..." ;
  prolstatus = elementProlongation();               // Create prolongation matrix.
  cout << "creating restriction matrix..." ;
  resstatus = elementRestriction();                 // Create restriction matrix

  if ( prolstatus == -1 ){
    cout << "ERROR: In initialising nested element Prolongation matrix!!" << endl;;
    exit(1);
  }
  if ( resstatus == -1 ){
    cout << "ERROR: In initialising nested element Restriction matrix!!" << endl;
    exit(1);
  }
  cout << "finished." << endl;

  el_prol_init = true;
  el_res_init = true;

  return 0;
}



int GridRelation::initNestElProlongation(){

  int prolstatus;

  cout << "Initializing nested element prolongation matrix..." ;
  prolstatus = elementProlongation();               // Create prolongation matrix.

  if ( prolstatus == -1 ){
    cout << "ERROR: In initialising nested element Prolongation matrix!!" << endl;;
    exit(1);
  }
  cout << "finished." << endl;

  el_prol_init = true;

  return 0;
}



int GridRelation::initNestElRestriction(){

  int prolstatus,resstatus;

  cout << "Initializing nested element restriction matrix..." ;
  resstatus = elementRestriction();                 // Create restriction matrix

  if ( resstatus == -1 ){
    cout << "ERROR: In initialising nested element Restriction matrix!!" << endl;
    exit(1);
  }
  cout << "finished." << endl;

  el_res_init = true;

  return 0;
}



int GridRelation::elementProlongation(){
  
  MV_Vector<real> midpointNode(nsd);
  MV_Vector<int> elNeighbors;
  MV_Vector<int> elmsTaken(fineNoElms);

  int noNodesInElm;
  int i,j,k,nbel;

  elmsTaken = 0.0;

  int *ncols;
  ncols = new int[fineNoElms];
  for ( int i; i < fineNoElms; i++ ) ncols[i] = 1;
  elProlMat = new SparseMatrix( fineNoElms, coarseNoElms, ncols );
  delete ncols;

  int el = 0;
  int EL = 0;

  for ( i=0; i < fineNoElms; i++ ){
    nextEl( el, elmsTaken, elNeighbors, finegrid ); 
    if ( noElms1 < noElms2 ){
      noNodesInElm = grid2->GetCell(el)->GetSize();
      for ( j=0; j < nsd; j++ ){
	midpointNode(j) = 0.0;
	for ( k=0; k < noNodesInElm; k++ ){
	  midpointNode(j) += grid2->GetNode(grid2->GetCell(el)->GetNode(k))->GetCoord(j);
	}
	midpointNode(j) /= noNodesInElm;
      } 
    }  
    else{
      noNodesInElm = grid1->GetCell(el)->GetSize();
      for ( j=0; j < nsd; j++ ){
	midpointNode(j) = 0.0;
	for ( k=0; k < noNodesInElm; k++ ){
	  midpointNode(j) += grid1->GetNode(grid1->GetCell(el)->GetNode(k))->GetCoord(j);
	}
	midpointNode(j) /= noNodesInElm;
      }
    }
    whatElement( midpointNode, EL, coarsegrid );
    elProlMat->Set(el,EL,0,1);
        
    // Take neighbor elements
    for ( nbel = 0; nbel < elNeighbors(elNeighbors.size()-1); nbel++ ){
      if ( elmsTaken( elNeighbors(nbel) ) == 0 ){
	i++;
	elmsTaken( elNeighbors(nbel) ) = 1; 
	el = elNeighbors(nbel);
	if ( noElms1 < noElms2 ){
	  noNodesInElm = grid2->GetCell(el)->GetSize();
	  for ( j=0; j < nsd; j++ ){
	    midpointNode(j) = 0.0;
	    for ( k=0; k < noNodesInElm; k++ ){
	      midpointNode(j) += grid2->GetNode(grid2->GetCell(el)->GetNode(k))->GetCoord(j);
	    }
	    midpointNode(j) /= noNodesInElm;
	  } 
	}  
	else{
	  noNodesInElm = grid1->GetCell(el)->GetSize();
	  for ( j=0; j < nsd; j++ ){
	    midpointNode(j) = 0.0;
	    for ( k=0; k < noNodesInElm; k++ ){
	      midpointNode(j) += grid1->GetNode(grid1->GetCell(el)->GetNode(k))->GetCoord(j);
	    }
	    midpointNode(j) /= noNodesInElm;
	  }
	}
	whatElement( midpointNode, EL, coarsegrid );
	elProlMat->Set(el,EL,0,1);
      }
    }
  }
  
  return 0;
}  


int GridRelation::elementRestriction(){

  int i,j,k,l;

  // Transpose elProlMat
  elResMat = elProlMat->GetCopy();
  elResMat->Transpose();

  // Normalise.
  real denom,aij;
  int col;
  for ( i=0; i < coarseNoElms; i++ ){
    denom = 0.0;
    for ( int pos=0; pos < elResMat->GetRowLength(i); pos++ ){
      aij = elResMat->Get(i,&col,pos);
      denom += aij;
    }
    for ( int pos=0; pos < elResMat->GetRowLength(i); pos++ ){
      aij = elResMat->Get(i,&col,pos);      
      elResMat->Set(i,col,pos,aij/denom);
    }
    if ( denom == 0.0 ){
      display->Error("denom = 0");
      exit(1);
    }
  }

  return 0;
}



int GridRelation::nextEl( int& el, MV_Vector<int>& elmsTaken, MV_Vector<int>& elNeighData, int whatGrid ){

  int i,j,k;
  int elNo;
  
  elNeighData.newsize(100);
  elNeighData = 0;
  
  if ( elmsTaken(el) == 0 ){
    elmsTaken(el) = 1;
    elNeighData(0) = el;
    if ( whatGrid == 1 ){
      elementNeighbors1( elNeighData );    
    }
    else{
      elementNeighbors2( elNeighData );
    }   
    return 0;
  }

  for (i=0; i < fineNoElms; i++){ 
    
    // If the search has got stuck, then take any node still 
    // not taken.
    if ( i > 6 ){
      //cout << "Long search" << endl;
      for ( j=0; j < fineNoElms; j++ ){
	if ( elmsTaken(j) == 0 ){
	  elmsTaken(j) = 1;
	  elNeighData(0) = el = j;
	  if ( whatGrid == 1 ){
	    elementNeighbors1( elNeighData );    
	  }
	  else{
	    elementNeighbors2( elNeighData );
	  }   
	  return 0;
	}  
      }
    }
    
    // Get neighbor nodes to prevNode in grid
    elNeighData(0) = el;
    if ( finegrid == 1 ){
      elementNeighbors1( elNeighData );    
    }
    else{
      elementNeighbors2( elNeighData );
    }   
    for (j=0; j < elNeighData(elNeighData.size()-1); j++){
      elNo = elNeighData(j);
      if ( elmsTaken(elNo) == 0){
	elmsTaken(elNo) = 1;
	elNeighData(0) = el = elNo;
	if ( whatGrid == 1 ){
	  elementNeighbors1( elNeighData );    
	}
	else{
	  elementNeighbors2( elNeighData );
	}   
	return 0;
      }  
    }
    
    // If all neighbor nodes are taken, take random neighbor node 
    // and check its neighbors.  
    real random = drand48();
    for ( k=0; k < elNeighData(elNeighData.size()-1); k++ ){
      if (  k >= random*elNeighData(elNeighData.size()-1) ){
	break;
      }
    }
    el = elNeighData(k);
  }
}




int GridRelation::projNodalData( MV_Vector<real> &val, MV_Vector<real> &projval ){

  int i,j;
  int fromNoNodes,toNoNodes;

  real aij;

  if ( val.size() == noNodes1 ){
    projval.newsize(noNodes2);
    fromNoNodes = noNodes1;
    toNoNodes = noNodes2;
  }
  else{
    projval.newsize(noNodes1);
    fromNoNodes = noNodes2;
    toNoNodes = noNodes1;
  }

  int col;
  if ( fromNoNodes < toNoNodes ){
    for ( i=0; i < toNoNodes; i++ ){
      projval(i) = 0.0;
      for (int pos=0;pos<prolMat->GetRowLength(i);pos++){
	aij = prolMat->Get(i,&col,pos);
	projval(i) += aij * val(col);
      }
    } 
  } else {
    for ( i=0; i < toNoNodes; i++ ){
      projval(i) = 0.0;
      for (int pos=0;pos<resMat->GetRowLength(i);pos++){
	aij = resMat->Get(i,&col,pos);
	projval(i) += aij * val(col);
      }
    }  
  }

  return 0;
}



int GridRelation::projNodalData( MV_ColMat<real> &val, MV_ColMat<real> &projval ){

  int i,j,d;
  int fromNoNodes,toNoNodes;

  real aij;

  if ( val.size(0) == noNodes1 ){
    projval.newsize(noNodes2,val.size(1));
    fromNoNodes = noNodes1;
    toNoNodes = noNodes2;
  }
  else{
    projval.newsize(noNodes1,val.size(1));
    fromNoNodes = noNodes2;
    toNoNodes = noNodes1;
  }

  int col;
  if ( fromNoNodes < toNoNodes ){
    for ( d=0; d < val.size(1); d++ ){    
      for ( i=0; i < toNoNodes; i++ ){
	projval(i,d) = 0.0;
	for (int pos=0;pos<prolMat->GetRowLength(i);pos++){
	  aij = prolMat->Get(i,&col,pos);
	  projval(i,d) += aij * val(col,d);
	}
      } 
    }
  }
  else {
    for ( d=0; d < val.size(1); d++ ){    
      for ( i=0; i < toNoNodes; i++ ){
	projval(i,d) = 0.0;
	for (int pos=0;pos<resMat->GetRowLength(i);pos++){
	  aij = resMat->Get(i,&col,pos);
	  projval(i,d) += aij * val(col,d);
	}
      } 
    }
  }

  return 0;
}





int GridRelation::projNodalData( MV_Vector<real> &val ){

  int projNoNodes;
  MV_Vector<real> projVal;

  //cout << "Start projecting nodal data...";

  if ( val.size() == coarseNoNodes ){
    projVal.newsize(fineNoNodes);
    projNoNodes = fineNoNodes;
  }
  else{
    if ( val.size() == fineNoNodes ){
      projVal.newsize(coarseNoNodes);
      projNoNodes = coarseNoNodes;
    }
    else{
      cout << "ERROR: Wrong number of nodes in vector!" << endl;
      exit(1);
    }
  }
  projNodalData( val, projVal ); 
  val.newsize(projNoNodes);
  int i;
  for ( i=0; i < projNoNodes; i++ ){
    val(i) = projVal(i);
  }
  //cout << "finished." << endl;

  return 0;
}
 



int GridRelation::projNodalData( MV_ColMat<real> &val ){

  int projNoNodes;
  MV_ColMat<real> projVal;
  int d = val.size(1);

  //cout << "Start projecting nodal data...";

  if ( val.size(0) == coarseNoNodes ){
    projVal.newsize(fineNoNodes,d);
    projNoNodes = fineNoNodes;
  }
  else{
    if ( val.size(0) == fineNoNodes ){
      projVal.newsize(coarseNoNodes,d);
      projNoNodes = coarseNoNodes;
    }
    else{
      cout << "ERROR: Wrong number of nodes in vector!" << endl;
      exit(1);
    }
  }
  projNodalData( val, projVal ); 
  val.newsize(projNoNodes,d);
  int i,j;
  for ( i=0; i < projNoNodes; i++ ){
    for ( j=0; j < d; j++ ){
      val(i,j) = projVal(i,j);
    }
  }
  //cout << "finished." << endl;

  return 0;
}
 

int GridRelation::projNestedElData( MV_Vector<real> &val, MV_Vector<real> &projval ){

  int i,j;
  int fromNoElms,toNoElms;

  real aij;

  if ( val.size() == noElms1 ){
    projval.newsize(noElms2);
    fromNoElms = noElms1;
    toNoElms = noElms2;
  }
  else{
    if ( val.size() == noElms2 ){
      projval.newsize(noElms1);
      fromNoElms = noElms2;
      toNoElms = noElms1;
    }
    else{
      cout << "ERROR: Wrong element size!!" << endl;
    }
  }

  //cout << "Start projecting nested element data..." << endl;

  int col;
  if ( fromNoElms < toNoElms ){
    for ( i=0; i < toNoElms; i++ ){
      projval(i) = 0.0;
      for (int pos=0;pos<elProlMat->GetRowLength(i);pos++){
	aij = elProlMat->Get(i,&col,pos);
	projval(i) += aij * val(col);
      }
    } 
  }
  else {
    for ( i=0; i < toNoElms; i++ ){
      projval(i) = 0.0;
      for (int pos=0;pos<elResMat->GetRowLength(i);pos++){
	aij = elResMat->Get(i,&col,pos);
	projval(i) += aij * val(col);
      }
    } 
  }

  return 0;
} 

int GridRelation::projNestedElData( MV_Vector<real>& val ){
  
  int projNoElms;
  MV_Vector<real> projval;

  if ( val.size() == coarseNoElms ){
    projval.newsize(fineNoElms);
    projNoElms = fineNoElms;
  }
  else{
    if ( val.size() == fineNoElms ){
      projval.newsize(coarseNoElms);
      projNoElms = coarseNoElms;
    }
    else{
      cout << "ERROR: Wrong number of elements in vector!" << endl;
      exit(1);
    }
  }

  projNestedElData( val, projval ); 
  val.newsize(projNoElms);
  int i;
  for ( i=0; i < projNoElms; i++ ){
    val(i) = projval(i);
  }

  return 0;
}



int GridRelation::projNestedElData( MV_ColMat<real> &val, MV_ColMat<real> &projval ){

  int i,j,d;
  int fromNoElms,toNoElms;

  real aij;

  if ( val.size(0) == noElms1 ){
    projval.newsize(noElms2,val.size(1));
    fromNoElms = noElms1;
    toNoElms = noElms2;
  }
  else{
    if ( val.size(0) == noElms2 ){
      projval.newsize(noElms1,val.size(1));
      fromNoElms = noElms2;
      toNoElms = noElms1;
    }
    else{
      cout << "ERROR: Wrong element size!!" << endl;
    }
  }

  //cout << "Start projecting nested element data..." << endl;
  
  int col;
  if ( fromNoElms < toNoElms ){
    for ( d=0; d < val.size(1); d++ ){
      for ( i=0; i < toNoElms; i++ ){
	projval(i,d) = 0.0;
	for (int pos=0;pos<elProlMat->GetRowLength(i);pos++){
	  aij = elProlMat->Get(i,&col,pos);
	  projval(i,d) += aij * val(col,d);
	}
      } 
    }
  }
  else {
    for ( d=0; d < val.size(1); d++ ){
      for ( i=0; i < toNoElms; i++ ){
	projval(i,d) = 0.0;
	for (int pos=0;pos<elResMat->GetRowLength(i);pos++){
	  aij = elResMat->Get(i,&col,pos);
	  projval(i,d) += aij * val(col,d);
	}
      } 
    }
  }

  return 0;
} 

int GridRelation::projNestedElData( MV_ColMat<real>& val ){
  
  int projNoElms;
  MV_ColMat<real> projval;

  if ( val.size(0) == coarseNoElms ){
    projval.newsize(fineNoElms,val.size(1));
    projNoElms = fineNoElms;
  }
  else{
    if ( val.size(0) == fineNoElms ){
      projval.newsize(coarseNoElms,val.size(1));
      projNoElms = coarseNoElms;
    }
    else{
      cout << "ERROR: Wrong number of elements in vector!" << endl;
      exit(1);
    }
  }

  projNestedElData( val, projval ); 
  val.newsize(projNoElms,projval.size(1));
  int i,d;
  for ( d=0; d < projval.size(1); d++ ){
    for ( i=0; i < projNoElms; i++ ){
      val(i,d) = projval(i,d);
    }
  }

  return 0;
}


int GridRelation::cpFieldRep( Field& field, Field& cpfield ){
  
  // This function copies a Field representation on one grid onto a nodal 
  // Field representation on the other grid.

  int cpNoNodes,cpgrid,grid;
  if ( field.GetNoNodes() == noNodes1 ){
    cpNoNodes = noNodes2;
    grid = 1;
    cpgrid = 2;
  }
  else{
    cpNoNodes = noNodes1;
    grid = 2;
    cpgrid = 1;
  }
  
  MV_Vector<real> cpvec(cpNoNodes);
  MV_Vector<real> point(nsd);

  int i,j,sd;
  int el = 0;
  MV_Vector<int> neighData(100);
  neighData = 0;
 
  MV_Vector<int> nodesTaken(cpNoNodes);
  for ( i=0; i < cpNoNodes; i++ ){
    nodesTaken(i) = 0;
  }

  int node = 0;
  whatElement( point, el, grid ); 
  cpvec(node) = field.Eval(point,el);
  nodesTaken(node)++;
  
  for ( i=0; i < cpNoNodes - 1; i++){ 
    nextNode( node, nodesTaken, neighData, cpgrid );
    for ( sd=0; sd < nsd; sd++ ){
      if ( cpgrid == 1 ){ 
	point(sd) = grid1->GetNode(node)->GetCoord(sd);
      }
      else{
	point(sd) = grid2->GetNode(node)->GetCoord(sd);
      }
    }
    whatElement( point, el, grid ); 
    cpvec(node) = field.Eval(point,el);
    for ( j=0; j < neighData(neighData.size()-1); j++){
      node = neighData(j);
      if ( nodesTaken(node) == 0 ){
	nodesTaken(node)++;
	i++;
	for ( sd=0; sd < nsd; sd++ ){
	  if ( cpgrid == 1 ){ 
	    point(sd) = grid1->GetNode(node)->GetCoord(sd);
	  }
	  else{
	    point(sd) = grid2->GetNode(node)->GetCoord(sd);
	  }
	}
	whatElement( point, el, grid ); 
	cpvec(node) = field.Eval(point,el);
      }      
    }
  }

  for ( i=0; i < cpNoNodes; i++ ){
    if ( nodesTaken(i) == 0 ){
      cout << "ERROR: Node " << i << " not taken." << endl;
      exit(1);
    }
    else{
      if ( nodesTaken(i) > 1 ){
	cout << "ERROR: Node " << i << " taken " << nodesTaken(i) << " times." << endl;
	exit(1);
      }
    }
  }
  
  cpfield.InitByNodes(&cpvec);

  return 0;
}  


int GridRelation::proj2PWConst( MV_Vector<real>& fineVal, MV_Vector<real>& coarseVal ){
  
  // Projects an element vector on the fine grid onto the space of piecewise 
  // constant functions on the coarse grid.

  if ( !(el_prol_init && el_res_init) ) initNestElMap();

  MV_Vector<real> projElVal(coarseNoElms);

  projNestedElData( fineVal, projElVal ); 
  projNestedElData( projElVal, coarseVal );

  return 0;
}



int GridRelation::trans2PWConst( MV_Vector<real>& fineVal, MV_Vector<real>& coarseVal ){
  
  // Projects an element vector on the fine grid onto the space of piecewise 
  // constant functions on the coarse grid.

  if ( !(el_prol_init && el_res_init) ) initNestElMap();

  projNestedElData( fineVal, coarseVal ); 

  return 0;
}



int GridRelation::proj2PWConst( Field& f ){
  
  // Projects a field on the fine grid (=grid1) onto the space of piecewise 
  // constant functions on the coarse grid (=grid2) .
  if ( finegrid != 1 ){
    cout<<"ERROR: Cannot project onto space of pw constants on a finer grid!!"<<endl;
    exit(1);
  }

  MV_Vector<real> elVal(fineNoElms);
  f.GetElementVector(elVal);
 
  MV_Vector<real> projElVal(coarseNoElms);

  initNestElMap();
  projNestedElData( elVal, projElVal ); 
  projNestedElData( projElVal, elVal );

  f.InitByElms( &elVal );

  return 0;
}



int GridRelation::proj2PWLin( Field& f ){
  
  // Projects a field on the fine grid (=grid1) onto the space of piecewise 
  // linear functions on the coarse grid (=grid2).
  if ( finegrid != 1 ){
    cout << "ERROR: Cannot project onto space of pw constants on a finer grid!!" << endl;
    exit(1);
  }

  MV_Vector<real> nodVal(fineNoNodes);
  f.GetNodalVector(nodVal);
  
  MV_Vector<real> projNodVal(coarseNoNodes);

  initNodalMap();
  projNodalData( nodVal, projNodVal ); 
  projNodalData( projNodVal, nodVal ); 
  
  f.InitByNodes( &projNodVal );

  return 0;
}



int GridRelation::avNodesOverElms( MV_Vector<real> &nodVal, MV_Vector<real> &elVal ){

  int noNodesInElm;
  int i,j;

  if ( nodVal.size() == noNodes1 ){
    elVal.newsize( noElms1 );
    for ( i=0; i < noElms1; i++ ){
      elVal(i) = 0.0;
      noNodesInElm = grid1->GetCell(i)->GetSize();
      for ( j=0; j < noNodesInElm; j++ ){
	elVal(i) += nodVal(grid1->GetCell(i)->GetNode(j));
      }
      elVal(i) /= noNodesInElm;
    }
  }
  else{
    if ( nodVal.size() == noNodes2 ){
      elVal.newsize( noElms2 );
      for ( i=0; i < noElms2; i++ ){
	elVal(i) = 0.0;
	noNodesInElm = grid2->GetCell(i)->GetSize();
	for ( j=0; j < noNodesInElm; j++ ){
	  elVal(i) += nodVal(grid2->GetCell(i)->GetNode(j));
	}
	elVal(i) /= noNodesInElm;
      }
    }
    else{
      cout << "ERROR: Wrong grid specification!!!" << endl;
      exit(1);
    }
  }

  return 0;
}  




int GridRelation::avNodesOverElms( MV_Vector<real> &Val ){

  MV_Vector<real> elVal;
  int noNodesInElm;
  int i,j;

  avNodesOverElms( Val, elVal );

  if ( Val.size() == noNodes1 ){
    Val.newsize( noElms1 );
    for ( i=0; i < noElms1; i++ ){
      Val(i) = elVal(i);
    }
  }
  else{
    if ( Val.size() == noNodes2 ){
      elVal.newsize( noElms2 );
      Val.newsize(noElms2);
      for ( i=0; i < noElms2; i++ ){
	Val(i) = elVal(i);
      }
    }
    else{
      cout << "ERROR: Wrong grid specification!!!" << endl;
      exit(1);
    }
  }
  
  return 0;
}  



int GridRelation::prolongation(){

  int *ncols;
  ncols = new int[fineNoNodes];
  if ( finegrid == 1 ){
    for ( int i; i < fineNoNodes; i++ ) ncols[i] = grid1->GetCell(i)->GetSize();
  } else{
    for ( int i; i < fineNoNodes; i++ ) ncols[i] = grid2->GetCell(i)->GetSize();
  }
  prolMat = new SparseMatrix( fineNoNodes, coarseNoNodes, ncols );
  delete ncols;

  int i,j; 
  int node,el,sd;

  MV_Vector<real> point(nsd);
  MV_Vector<int> nodeNeighData(100);
  MV_Vector<int> nodesTaken(fineNoNodes);
 
  nodesTaken = 0;
  baryCoord = 0;
  nodeNeighData = 0;

  node = 0;

  if ( finegrid == 1 ){
    for ( sd=0; sd < nsd; sd++ ){
      point(sd) = grid1->GetNode(node)->GetCoord(sd);
    }
  }
  else{
    for ( sd=0; sd < nsd; sd++ ){
      point(sd) = grid2->GetNode(node)->GetCoord(sd);
    }
  }

  el = 0;
  whatElement( point, el, coarsegrid );
  if ( el < 0 ){
    cout << "ERROR 1: Node is not in any element!!!" << endl; 
    exit(1);
  }
  updateProl( node, el ); 
  nodesTaken(node)++;
  for ( i=0; i < fineNoNodes-1; i++ ){
    nextNode( node, nodesTaken, nodeNeighData, finegrid );
    if ( finegrid == 1 ){
      for ( sd=0; sd < nsd; sd++ ){
	point(sd) = grid1->GetNode(node)->GetCoord(sd);
      }
    }
    else{
      for ( sd=0; sd < nsd; sd++ ){
	point(sd) = grid2->GetNode(node)->GetCoord(sd);
      }
    }
    whatElement( point, el, coarsegrid );
    if ( el < 0 ){
      cout << "ERROR 2: Node is not in any element!!!" << endl; 
      exit(1);
    }
    updateProl( node, el ); 
    for ( j=0; j < nodeNeighData(nodeNeighData.size()-1); j++ ){
      node = nodeNeighData(j);
      if ( nodesTaken(node) == 0 ){
	if ( finegrid == 1 ){
	  for ( sd=0; sd < nsd; sd++ ){
	    point(sd) = grid1->GetNode(node)->GetCoord(sd);
	  }
	}
	else{
	  for ( sd=0; sd < nsd; sd++ ){
	    point(sd) = grid2->GetNode(node)->GetCoord(sd);
	  }
	}
	i++;
	nodesTaken(node)++;      
	whatElement( point, el, coarsegrid );
	if ( el < 0 ){
	  cout << "ERROR 3: Node is not in any element!!!" << endl; 
	  exit(1);
	}
 	updateProl( node, el ); 
      }
    }
  }

  // Control so that all nodes in fine grid are taken.
  for ( i=0; i < fineNoNodes; i++ ){
    if ( nodesTaken(i) == 0 ){
      printf("ERROR: Node %i in fine grid not taken\n",i);
    }
    if ( nodesTaken(i) > 1 ){
      printf("ERROR: Node %i in fine grid taken %i times\n",i,nodesTaken(i));
    }
  }

  return 0;
}



int GridRelation::restriction(){

  int i,j,k,l;

  // Transpose prolMat
  resMat = prolMat->GetCopy();
  resMat->Transpose();
  
  /*
  MV_Vector<int> columns(coarseNoNodes);
  columns(i) = 0;
  for ( i=0; i < fineNoNodes; i++ ){
    for ( j=0; j < prolMat.irow(i+1) - prolMat.irow(i); j++ ){
      columns( prolMat.jcol(prolMat.irow(i) + j) ) ++; 
    }
  }          
                         
  resMat.irow(0) = 0;
  for ( i=0; i < coarseNoNodes-1; i++ ) resMat.irow(i+1) = resMat.irow(i) + columns(i);
  //  resMat.irow(coarseNoNodes) = prolMat.GetNoNonzeroes() + 1;
  resMat.irow(coarseNoNodes) = prolMat.GetNoNonzeroes();

  columns(i) = 0;
  for( i=0; i < fineNoNodes; i++ ){
    for ( j=0; j < prolMat.irow(i+1) - prolMat.irow(i); j++ ){
      resCoeff(resMat.irow(prolMat.jcol(prolMat.irow(i)+j))+columns(prolMat.jcol(prolMat.irow(i)+j))) = prolCoeff(prolMat.irow(i)+j); 
      resMat.jcol(resMat.irow(prolMat.jcol(prolMat.irow(i)+j))+columns(prolMat.jcol(prolMat.irow(i)+j))) = i;
      columns(prolMat.jcol(prolMat.irow(i)+j))++;
    }
  }
  */

  // Normalise.
  real denom,aij;
  int col;

  for ( i=0; i < coarseNoElms; i++ ){
    denom = 0.0;
    for ( int pos=0; pos < elResMat->GetRowLength(i); pos++ ){
      aij = elResMat->Get(i,&col,pos);
      denom += aij;
    }
    for ( int pos=0; pos < elResMat->GetRowLength(i); pos++ ){
      aij = elResMat->Get(i,&col,pos);      
      elResMat->Set(i,col,pos,aij/denom);
    }
    if ( denom == 0.0 ){
      display->Error("denom = 0");
      exit(1);
    }
  }

  return 0;
}
  


int GridRelation::nextNode( int& node, 
			    MV_Vector<int>& nodesTaken, 
			    MV_Vector<int>& neighData, 
			    int whatGrid ){

  int i,j,k;
  int nodeNo;

  neighData = 0;

  for (i=0; i < fineNoNodes; i++){ 

    // If the search has got stuck, then take any node still 
    // not taken.
    if ( i > 6 ){
      //cout<<"Long search"<<endl;
      for ( j=0; j < fineNoNodes; j++ ){
	if ( nodesTaken(j) == 0 ){
	  nodesTaken(j)++;
	  neighData(0) = node = j;
	  if ( whatGrid == 1 ){
	    nodeNeighbors1( neighData );    
	  }
	  else{
	    nodeNeighbors2( neighData );
	  }   
	  return 0;
	}  
      }
    }
    // Get neighbor nodes to prevNode in grid
    neighData(0) = node;
    if ( finegrid == 1 ){
      nodeNeighbors1( neighData );    
    }
    else{
      nodeNeighbors2( neighData );
    }   
    for (j=0; j < neighData(neighData.size()-1); j++){
      nodeNo = neighData(j);
      if (nodesTaken(nodeNo) == 0){
	nodesTaken(nodeNo)++;
	neighData(0) = node = nodeNo;
	if ( whatGrid == 1 ){
	  nodeNeighbors1( neighData );    
	}
	else{
	  nodeNeighbors2( neighData );
	}   
	return 0;
      }  
    }
    
    // If all neighbor nodes are taken, take random neighbor node 
    // and check its neighbors.  
    real random = drand48();
    for ( k=0; k < neighData(neighData.size()-1); k++ ){
      if (  k >= random*neighData(neighData.size()-1) ){
	break;
      }
    }
    node = neighData(k);
  }
}



int GridRelation::nodeNeighbors1( MV_Vector<int>& neighData ){

  int i,j;

  int node = neighData(0);

  Node *n;
  n = grid1->GetNode(node);
  
  j=0;
  // Set neighData() to nodes excluding node  
  for (i=0; i < n->GetNoNodeNeighbors(); i++){
    if ( n->GetNodeNeighbor(i) != node ){
      neighData(j) = n->GetNodeNeighbor(i);
      j++;
    }
  }
  // neighData(neighData.size()-1) is number of neighbors (node excluded)
  neighData(neighData.size()-1) = n->GetNoNodeNeighbors()-1; 

  return 0;
}



int GridRelation::nodeNeighbors2( MV_Vector<int>& neighData ){

  int i,j;

  int node = neighData(0);

  Node *n;
  n = grid2->GetNode(node);
  
  j=0;
  // Set neighData() to nodes excluding node  
  for (i=0; i < n->GetNoNodeNeighbors(); i++){
    if ( n->GetNodeNeighbor(i) != node ){
      neighData(j) = n->GetNodeNeighbor(i);
      j++;
    }
  }
  // neighData(neighData.size()-1) is number of neighbors (node excluded)
  neighData(neighData.size()-1) = n->GetNoNodeNeighbors()-1; 

  return 0;
}


int GridRelation::elementNeighbors1( MV_Vector<int>& neighborElms ){

  int i; 
  
  int elem = neighborElms(0);

  Cell *c;
  c = grid1->GetCell(elem);

  for (i=0; i < c->GetNoCellNeighbors(); i++){
    neighborElms(i) = c->GetCellNeighbor(i);
  }
  neighborElms(neighborElms.size()-1) = c->GetNoCellNeighbors(); 

  return 0;
}


int GridRelation::elementNeighbors2( MV_Vector<int>& neighborElms ){

  int i; 
  
  int elem = neighborElms(0);

  Cell *c;
  c = grid2->GetCell(elem);

  for (i=0; i < c->GetNoCellNeighbors(); i++){
    neighborElms(i) = c->GetCellNeighbor(i);
  }
  neighborElms(neighborElms.size()-1) = c->GetNoCellNeighbors(); 

  return 0;
}



int GridRelation::GetBarycentric( MV_Vector<real>& point, int el, int whatGrid ){

  int i,noBC,err;
  
  if ( whatGrid == 1 ){
    noBC = grid1->GetCell(el)->GetSize();
  }
  else{
    if ( whatGrid == 2 ){
      noBC = grid2->GetCell(el)->GetSize();
    }
    else{
      cout << "ERROR: Wrong grid specified!!!" << endl;
    }
  }

  if ( noBC == 0 ){
    cout << "Ohh, nsd, noBC, el: " << nsd << ", " << noBC << ", " << el << endl;
    exit(1);
  }

  if ( nsd == 2 ){
    if (noBC == 3 ){
      err = GetBarycentricTri( point, el, whatGrid );
    }
    else{
      if ( noBC == 4 ){
	err = GetBarycentricQuad( point, el, whatGrid );	
	if ( err == 1 ){
	  return 1;
	}
      }
      else{
	printf("ERROR: This element type is not implemented!!!\n");
	exit(1);
      }
    }  
  }
  else{
    if ( nsd == 3 ){
      if (noBC == 4 ){
	err = GetBarycentricTet( point, el, whatGrid );
      }
      else{
	if ( noBC == 8 ){
	  err = GetBarycentricHex( point, el, whatGrid );
	}
	else{
	  printf("ERROR: This element type is not implemented!!!\n");
	  exit(1);
	}
      }
    }
    else{
      printf("ERROR: Wrong number of space dimensions!!!\n");
      exit(1);
    }
  }
  if ( err == -1 ){
    printf("ERROR: In computing barycentric coordinates!!!\n");
    exit(1);
  }

  return noBC;
}




int GridRelation::GetBarycentricTri( MV_Vector<real>& point, int el, int whatGrid ){

  /*
  FET3n2D* FE=NULL;
  if ( whatGrid == 1 ){
    FE = &grid1->FET3;
  }
  else{
    if ( whatGrid == 2 ){
      FE = &grid2->FET3;
    }
    else{
      printf("ERROR: Wrong grid specified!!!\n");
      exit(1);
    }
  }
  FE->refill( el );
  baryCoord(0) = FE->barycentric1( point(0), point(1) );
  baryCoord(1) = FE->barycentric2( point(0), point(1) );
  baryCoord(2) = FE->barycentric3( point(0), point(1) );
  */

  return 0;
}




int GridRelation::GetBarycentricQuad( MV_Vector<real>& globpt, int el, int whatGrid ){

  MV_ColMat<real> coor(4,2);

  // ATTENTION: In the functions .GetBarycentricQuad() & .canBeInsideBox(), 
  //            the numbering of the local nodes is not the same:
  //
  // Here:      3----4      Elsewhere:      4----3 
  //            |    |                      |    |
  //            |    |                      |    | 
  //            1----2                      1----2  

  if ( whatGrid == 1 ){
    coor(0,0) = grid1->GetNode(grid1->GetCell(el)->GetNode(0))->GetCoord(0);
    coor(0,1) = grid1->GetNode(grid1->GetCell(el)->GetNode(0))->GetCoord(1);
    coor(1,0) = grid1->GetNode(grid1->GetCell(el)->GetNode(1))->GetCoord(0);
    coor(1,1) = grid1->GetNode(grid1->GetCell(el)->GetNode(1))->GetCoord(1);
    coor(2,0) = grid1->GetNode(grid1->GetCell(el)->GetNode(3))->GetCoord(0);
    coor(2,1) = grid1->GetNode(grid1->GetCell(el)->GetNode(3))->GetCoord(1);
    coor(3,0) = grid1->GetNode(grid1->GetCell(el)->GetNode(2))->GetCoord(0);
    coor(3,1) = grid1->GetNode(grid1->GetCell(el)->GetNode(2))->GetCoord(1);
  }
  else{
    if ( whatGrid == 2 ){
      coor(0,0) = grid2->GetNode(grid2->GetCell(el)->GetNode(0))->GetCoord(0);
      coor(0,1) = grid2->GetNode(grid2->GetCell(el)->GetNode(0))->GetCoord(1);
      coor(1,0) = grid2->GetNode(grid2->GetCell(el)->GetNode(1))->GetCoord(0);
      coor(1,1) = grid2->GetNode(grid2->GetCell(el)->GetNode(1))->GetCoord(1);
      coor(2,0) = grid2->GetNode(grid2->GetCell(el)->GetNode(3))->GetCoord(0);
      coor(2,1) = grid2->GetNode(grid2->GetCell(el)->GetNode(3))->GetCoord(1);
      coor(3,0) = grid2->GetNode(grid2->GetCell(el)->GetNode(2))->GetCoord(0);
      coor(3,1) = grid2->GetNode(grid2->GetCell(el)->GetNode(2))->GetCoord(1);
    }
    else{
      cout << "ERROR: Wrong grid specified!!!" << endl;
      exit(1);
    }
  }

  if( canBeInsideBox(globpt,coor,4,0) == -1 ){ // straight sides
    return 1;
  }

  // solve for local point, using Newton's method
  const int  max_iterations = 20;    // max no of Newton iterations
  const real eps = 1.0e-10;          // max error in Newton's method
  int niter = 0;
  real error = 1e+20;
  
  // start point is the centroid of the element, here (0,0)
  
  real x1 = coor(0,0), y1 = coor(0,1);
  real x2 = coor(1,0), y2 = coor(1,1);
  real x3 = coor(2,0), y3 = coor(2,1);
  real x4 = coor(3,0), y4 = coor(3,1);
  
  real x21 = x2-x1,  x43 = x4-x3,  x31 = x3-x1,  x42 = x4-x2;
  real y21 = y2-y1,  y43 = y4-y3,  y31 = y3-y1,  y42 = y4-y2;
  
  real f1 = 4*globpt(0) - (x1+x2+x3+x4);
  real f2 = 4*globpt(1) - (y1+y2+y3+y4);
  
  real j11 = x21 + x43,  j12 = x31 + x42;
  real j21 = y21 + y43,  j22 = y31 + y42;
  
  real invdet = 1./(j11*j22 - j12*j21);
  
  real r,s,r0,s0,dr,ds;
  r0 = r = (j22*f1 - j12*f2)*invdet;
  s0 = s = (j11*f2 - j21*f1)*invdet;
  
  f1 = x43 - x21;
  f2 = y43 - y21;
  
  real d1 = (j22*f1 - j12*f2)*invdet;
  real d2 = (j11*f2 - j21*f1)*invdet;

  while ( niter++ < max_iterations && error > eps ){
    dr = r0/( 1 + d1*s ) - r;
    r += dr;
    ds = s0/( 1 + d2*r ) - s;
    s += ds;
    error = sqr(dr) + sqr(ds);
    //    cout<<r<<" "<<s<<" "<<error<<endl;
  }
  if(niter >= max_iterations){
    //    warningFP("ElmB4n2d:: findLocPt","Iteration did not converge");
    cout << "ERROR: Newton iteration didn't converge!!!" << endl; 
    cout <<" iter="<<niter<<" dr="<<dr<<" ds="<<ds<<endl;
    cout <<" d1="<<d1<<" d2="<<d1<<" r0="<<r0<<" s0="<<s0<<endl;
    //coor.print(s_o);
  }
  // check if locpt is valid 
  //  locpt.redim(2);
  //  locpt(1) = r;
  //  locpt(2) = s;
  baryCoord(0) = (0.5-0.5*r)*(0.5-0.5*s);
  baryCoord(1) = (0.5+0.5*r)*(0.5-0.5*s);
  baryCoord(2) = (0.5+0.5*r)*(0.5+0.5*s);
  baryCoord(3) = (0.5-0.5*r)*(0.5+0.5*s);
  error = sqrt(error);
  if ( r > 1.001 ||  r < -1.001 ||  s > 1.001 ||  s < -1.001 )
    return 1;
  return 0;
//  return isInside (locpt);
}

  
//-----------------------------------------------------------------------------
//BooLean ElmDef:: canBeInsideBox
//  (const Ptv(real)& globpt, const Mat(real)& coor, BooLean curved) const
//-----------------------------------------------------------------------------
//{
  // default version that can be inherited for some box-type elements


int GridRelation::canBeInsideBox( MV_Vector<real>& globpt, MV_ColMat<real>& coor, int noNodes, int curved ){

  //  BooLean inside = dpTRUE;
  int inside = 1;
  real min,max,x,diff,gbpt;
  int i,j,h;
  real comparison_tolerance = 0.01;

  //  h = coor.rows();
  for (i = 0; i < nsd; i++){
      min = max = coor(0,i);
      for (j = 1; j < noNodes; j++)
        {
          x = coor(j,i);
          if (x < min) min = x;
          else
            if (x > max) max = x;
        }
      if ( curved == 1 )
        {
          diff = (max-min)*0.2;
          max += diff;
          min -= diff;
        }
      gbpt = globpt(i);
      if ( max  + comparison_tolerance < gbpt ||
           gbpt + comparison_tolerance < min )
        {
	  //	  inside = dpFALSE;
	  inside = -1;
          break;
        }

    }
  return inside;
}




int GridRelation::GetBarycentricTet( MV_Vector<real>& point, int el, int whatGrid )
{
 
  display->Error("GridRelation::GetBaryCentricTet()","Not implemented");
  
  /*
	 ShapeFunctionTetLin_old *sfcn;
	 
  if ( whatGrid == 1 ){
  sfcn = new ShapeFunctionTetLin_old(grid1);
  }
  else{
  if ( whatGrid == 2 ){
  sfcn = new ShapeFunctionTetLin_old(grid2);
  }
  else{
  printf("ERROR: Wrong grid specified!!!\n");
  return -1;
  }
  }
  sfcn->Refill(el);
  baryCoord(0) = sfcn->Barycentric1( point(0), point(1), point(2) );
  baryCoord(1) = sfcn->Barycentric2( point(0), point(1), point(2) );
  baryCoord(2) = sfcn->Barycentric3( point(0), point(1), point(2) );
  baryCoord(3) = sfcn->Barycentric4( point(0), point(1), point(2) );
  */
  
  return 0;
}
  


int GridRelation::GetBarycentricHex( MV_Vector<real>& point, int el, int whatGrid ){
  /*

  FET8n3D* FE=NULL;
  if ( whatGrid == 1 ){
    FE = &grid1->FET8;
  }
  else{
    if ( whatGrid == 2 ){
    FE = &grid2->FET8;
    }
    else{
      printf("ERROR: Wrong grid specified!!!\n");
    }
  }
  FE->refill( el );
  baryCoord(0) = FE->barycentric1(x,y,z);
  baryCoord(1) = FE->barycentric2(x,y,z);
  baryCoord(2) = FE->barycentric3(x,y,z);
  baryCoord(3) = FE->barycentric4(x,y,z);
  baryCoord(4) = FE->barycentric5(x,y,z);
  baryCoord(5) = FE->barycentric6(x,y,z);
  baryCoord(6) = FE->barycentric7(x,y,z);
  baryCoord(7) = FE->barycentric8(x,y,z);
  */
  return 0;
}
  

int GridRelation::updateProl( int node, int el ){
  
  if ( coarsegrid == 1 ){
    for ( int pos=0; pos < grid1->GetCell(el)->GetSize(); pos++ ){
      prolMat->Set(node,grid1->GetCell(el)->GetNode(pos),pos,baryCoord(pos));
    }
  }
  else{
    for ( int pos=0; pos < grid2->GetCell(el)->GetSize(); pos++ ){
      prolMat->Set(node,grid2->GetCell(el)->GetNode(pos),pos,baryCoord(pos));
    }
  }

  return 0;
}




int GridRelation::whatElement( MV_Vector<real>& point, int& el, int whatGrid ){

  int i,j,k;
  real random;
  int noElms,nodeInElm;
  MV_Vector<int> neighborElms(100);
  
  if ( whatGrid == 1 ){
    noElms = noElms1;
  }
  else{
    if ( whatGrid == 2 ){
      noElms = noElms2;
    }
    else{
      cout << "ERROR: Wrong grid specified!" << endl;
      exit(1);
    }
  }

  for( i=0; i < noElms; i++){  
    // If search got stuck, search through all elements
    if ( i > 10 ){
      //cout << "Long Element search..." << endl;
      for ( j=0; j < noElms; j++ ){
	el = j;
	//cout << "Check if node is in element" << endl;
	nodeInElm = checkIfInElm( point, el, whatGrid );
	if ( nodeInElm == 1 ){
	  return 0;
	}
      }
      //cout << "ERROR 4: Node is not in any element!!!" << endl;
      exit(1);
    }
    //cout << "Check if node is in element" << endl;
    nodeInElm = checkIfInElm( point, el, whatGrid );
    if ( nodeInElm == 1 ){
      return 0;
    }
    //cout << "If node is not in el, check its neighbor elements" << endl;
    neighborElms(0) = el;
    if ( whatGrid == 1 ){
     elementNeighbors1( neighborElms );
    }
    else{
      elementNeighbors2( neighborElms );
    }
    //cout << "Loop over neighbor elements in grid2" << endl;
    for( j = 0; j < neighborElms(neighborElms.size()-1); j++){
      el = neighborElms(j);
      //cout << "Check if node is in element" << endl;
      nodeInElm = checkIfInElm( point, el, whatGrid );
      if ( nodeInElm == 1 ){
	return 0;
      }
    } 
    // If node is not in el or its neighbors, check 
    // neighbors to a random neighbor element.  
    random = drand48();
    for ( k=0; k < neighborElms(neighborElms.size()-1); k++ ){
      if ( k >= random*neighborElms(neighborElms.size()-1) ){
	break;
      }
    }
    el = neighborElms(k-1);
  }
}
  


int GridRelation::checkIfInElm( MV_Vector<real>& point, int el, int whatGrid ){

  int i;
  int noBaryC,inElm;

  noBaryC = GetBarycentric( point, el, whatGrid );

  if ( noBaryC == -1 ){
    printf("ERROR: Couldn't Get barycentric coordinates!!!\n");
    exit(1);
  }
  if ( noBaryC == 1 ){
    // node not in element.
    return 0;
  }
  // Check if node is in el
  inElm = 1;
  for ( i=0; i < noBaryC; i++ ){
   if ( baryCoord(i) < 0 ){
     inElm =0;
   }
  }
  if ( inElm == 1 ){ 
   return 1;
  }
  else{
    return 0;
  }
}


GridRelation::~GridRelation(){

  cout << "GridRelation destroyed..." << endl;
}
  


/* 
   See GridRelation.h for more information. 
   If you have questions, contact: hoffman@math.chalmers.se

   Written by Johan Hoffman, Chalmers Finite Element Center 1999 (c).
*/



