// Copyright (C) 2002 Johan Hoffman and Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Original author: Johan Hoffman. Added to Kraftwerk 2002-02-05.

#ifndef __LOCALFIELD_HH
#define __LOCALFIELD_HH

#include <kw_constants.h>

class FunctionSpace;
class ShapeFunction;
class GlobalField;
class FiniteElement;

class LocalField {
  
public:

  LocalField();
  LocalField(FiniteElement *element);

  ~LocalField();

  void AttachFiniteElement(FiniteElement *element);

  void operator = (LocalField &lf);

  void Refill(int cellnumber, real *nvals);
  void Refill(int cellnumber, int component, GlobalField *field);

  void Add(LocalField &lf);
  void Mult(real a);
  
  void EvalGradient(real* gradient); 
  void ComputeGradient(); 
  real EvalGradient(int component); 

  real dx();
  real dy();
  real dz();

  int GetDim() const;

  int  GetCellNo() const;
  real GetCoord(int node, int dim) const;
  real GetDofValue(int dof) const;
  real GetMeanValue() const;

  FiniteElement* GetFiniteElement() const;
  FunctionSpace* GetFunctionSpace();
  const ShapeFunction* GetShapeFunction(int i) const; 


protected:

  real *dof_values;
  real *gradient;
  
  int dim;
  
  bool gradient_computed;

  FunctionSpace *functionspace;

};

#endif
