#ifndef QuadratureRules_h
#define QuadratureRules_h

#include "kw_constants.h"
#include <mvvtp.h>
#include <mvvind.h>
#include <mvmtp.h>

enum QuadGeomType
{
  NO_QUADGEOM_TYPE = 0,// default dummy for not implemented element
  LINE,
  TRI,
  QUAD,
  TET,
  HEX,
  PYR,
  PRISM,
  SIMPLEX,
  HYPERCUBE
};

class QuadratureRules {

public:
  QuadratureRules ();
  QuadratureRules( const QuadGeomType& quadtype_, const int order_ );

  QuadratureRules( const QuadGeomType& quadtype_, 
		   const int order_, 
		   const int dimension );

  QuadratureRules( const MV_ColMat<real>& points_, 
		   const MV_Vector<real>& weights_, 
		   const QuadGeomType quadtype_ = NO_QUADGEOM_TYPE,
		   const int order_ = -1 );

  ~QuadratureRules(){};

  int init( const QuadGeomType& quadtype_, const int order_ );

  void getQuadratureRule ( MV_ColMat<real>& points_, 
			   MV_Vector<real>& weights_ ) const ;

  int getOrder() const { return order; }

  QuadGeomType getGeomType() const { return quadtype; }

  real getDimension() const { return points.size(1); }

  int getNoQuadraturePnts() const { return weights.size(); }

  real getQuadraturePnt( const int qpnt, const int dim ) const
    { return points(qpnt,dim); }
 
  const MV_ColMat<real>& getQuadraturePnt () const { return points; }

  real getWeight ( const int qpnt ) const { return weights(qpnt); }

  const MV_Vector<real>& getWeight () const { return weights; }

private:

  QuadGeomType  quadtype; 

  int order;

  MV_ColMat<real> points;
  MV_Vector<real> weights;

};

#endif
