// Original author: Johan Hoffman. Added to Kraftwerk 2002-01-14.
// Copyright (C) 2002 Johan Hoffman and Anders Logg.
// Licensed under the GNU GPL Version 2.

#ifndef __STAT_PROBLEM_HH
#define __STAT_PROBLEM_HH

#include <kw_constants.h>
#include <Settings.hh>
#include <Globals.hh>
#include <SparseMatrix.hh>
#include <Vector.hh>
#include <SISolver.hh>
#include <KrylovSolver.hh>
#include "Field.hh"
#include "Equation.hh"
#include "Discretiser.hh"
#include "GridRelation.h"
#include "UtilFE.h"


// FIXME: Removed until we get the Grid class working
//#include "MultigridUtilities.hh"

extern Settings *settings;
extern Globals *globals;

class StatProblem{

 public:
  
  /// Constructor
  StatProblem(Grid *grid); 
  
  /// Destructor
  ~StatProblem();

  /// Set internal pointers to solution vectors
  int Set_uk( MV_ColMat<real>* uld );  
  int Set_uk_prev_TS( MV_ColMat<real>* uld );
  int Set_uk_prev_NL( MV_ColMat<real>* uld );
  int Set_Uprimal( MV_ColMat<real>* uld );

  /// Time step problem
  virtual int Timestep( MV_ColMat<real> &prevVal, real time, real dt ) = 0;

  /// Returns a pointer to the Grid associated to StatProblem
  Grid *GetGrid();

  /// Returns a pointer to the Grid vector associated to StatProblem
  Grid* GetCompGrid();

  /// Returns number of equations
  int GetNoEq();

  /// Attach finite element discretization utilities
  int AttachFEM(Discretiser* dsc, Equation* eqn);

  /// Set internal pointers to multiscale vectors 
  int SetMultiScale( MV_ColMat<real>* uc, MV_ColMat<real>* uf );
  

 protected:
  
  /// finite element discretization utilities
  Discretiser* discrete;
  Equation* equation;

  //Mapper* mapper;
  //QuadratureRules* quadrule;
  //Element* element;
  
  /// exact integration or quadrature
  bool exact_integration;

  /// Pointer to the grid
  Grid *grid;
  
  MV_Vector<Field*> data;

  MV_ColMat<real>* uk;
  MV_ColMat<real>* uk_prev_TS;
  MV_ColMat<real>* uk_prev_NL;
  MV_ColMat<real>* Uprimal;

  bool compute_projections;
  MV_ColMat<real>* uk_coarse;
  MV_ColMat<real>* uk_fine;
  
  int noNodes;
  int noElms;
  int nsd;
  int noeq;

  SparseMatrix* A;

  //MV_ColMat<Mat> A_mat;
  //MV_ColMat<Mat> GridProjections;
  bool A_assembled; 
  bool F_assembled; 

  int compGrid;
  
  // FIXME: Should be removed
  bool debug_mode;

};

#endif
