#include "UtilFE.h"
#include "Settings.hh"

extern Settings *settings;

//-----------------------------------------------------------------------------
int getBndNodes(MV_Vector<int>& bndNodes, int* noBndNodes, MV_ColMat<real>& bndLim, Grid* grid)
{
  int cnt = 0;
  int i,j;
  int nsd;
  Node *n;
  
  settings->Get("dimension",&nsd);

  MV_Vector<int> tmp( grid->GetNoNodes() );
  switch(nsd){
  case 1:
    for ( i=0; i < grid->GetNoNodes(); i++ ){
      n = grid->GetNode(i);
      if ( n->GetCoord(0) >= bndLim(0,0) && n->GetCoord(0) <= bndLim(0,1) ){
	tmp(cnt) = i;
	cnt++;
      }
    }
    bndNodes.newsize(cnt);
    for ( i=0; i < cnt; i++ ){
      bndNodes(i) = tmp(i);
    }
    break;
  case 2:
    for ( i=0; i < grid->GetNoNodes(); i++ ){
      n = grid->GetNode(i);
      if ( n->GetCoord(0) >= bndLim(0,0) && n->GetCoord(0) <= bndLim(0,1) && 
	   n->GetCoord(1) >= bndLim(1,0) && n->GetCoord(1) <= bndLim(1,1) ){
	tmp(cnt) = i;
	cnt++;
      }
    }
    bndNodes.newsize(cnt);
    for ( i=0; i < cnt; i++ ){
      bndNodes(i) = tmp(i);
    }
    break;
  case 3:
    for ( i=0; i < grid->GetNoNodes(); i++ ){
      n = grid->GetNode(i);
      if ( n->GetCoord(0) >= bndLim(0,0) && n->GetCoord(0) <= bndLim(0,1) && 
	   n->GetCoord(1) >= bndLim(1,0) && n->GetCoord(1) <= bndLim(1,1) && 
	   n->GetCoord(2) >= bndLim(2,0) && n->GetCoord(2) <= bndLim(2,1) ){	
	tmp(cnt) = i;
	cnt++;
      }
    }
    bndNodes.newsize(cnt);
    for ( i=0; i < cnt; i++ ){
      bndNodes(i) = tmp(i);
    }
    break;
  default:
    printf("ERROR: Wrong number of space dimensions!!\n");
    exit(1);
    break; 
  }
  *noBndNodes = cnt;
  
  return 0;
}
//-----------------------------------------------------------------------------
/*
int dirichletBC( Mat A, Vec b, MV_Vector<int>& nodeList, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int *bndNodes; 
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  int noBndNodes = nodeList.size();
  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i]  = nodeList(i) * nsys;
    bndNodes[i] += eq;
  }
 
  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCVal.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  IS bndRows;
  ierr = ISCreateGeneral(PETSC_COMM_SELF, noBndNodes, bndNodes, &bndRows);
  ierr = MatZeroRows( A, bndRows, &one); CHKERRQ(ierr);

  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = value;
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );

  ierr = PetscFree( bndNodes );
  ierr = ISDestroy( bndRows );
  PetscFunctionReturn(0);
}
//-----------------------------------------------------------------------------
int dirichletBC( Vec b, MV_Vector<int>& nodeList, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int *bndNodes; 
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  int noBndNodes = nodeList.size();
  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = nodeList(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = value;
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );
 
  ierr = PetscFree( bndNodes );
  PetscFunctionReturn(0);
}
//-----------------------------------------------------------------------------
int dirichletBC( Mat A, Vec b, int node, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int *bndNodes; 
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  int noBndNodes = 1;
  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i]  = node * nsys;
    bndNodes[i] += eq;
  }
 
  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCVal.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  IS bndRows;
  ierr = ISCreateGeneral(PETSC_COMM_SELF, noBndNodes, bndNodes, &bndRows);
  ierr = MatZeroRows( A, bndRows, &one); CHKERRQ(ierr);

  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = value;
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );

  ierr = PetscFree( bndNodes );
  ierr = ISDestroy( bndRows );
  PetscFunctionReturn(0);
}
//-----------------------------------------------------------------------------
int dirichletBC( Vec b, int node, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int *bndNodes; 
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  int noBndNodes = 1;
  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = node * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = value;
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );
 
  ierr = PetscFree( bndNodes );
  PetscFunctionReturn(0);
}
*/
//-----------------------------------------------------------------------------
int dirichletBC( SparseMatrix* A, Vector* b, MV_ColMat<real>& bndLim, real value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  MV_Vector<int> bNodes( grid->GetNoNodes() );
  int noBndNodes;
  double *diriVal;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);
  
  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }
 
  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCVal.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  A->SetRowIdentity(bndNodes,noBndNodes);
  
  for( i = 0; i < noBndNodes; i++) b->Set(bndNodes[i],value);

  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;

}
//-----------------------------------------------------------------------------
int dirichletBC( Vector* b, MV_ColMat<real>& bndLim, real value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  MV_Vector<int> bNodes( grid->GetNoNodes() );
  int noBndNodes;
  double *diriVal;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);
  
  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }
 
  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCVal.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  for( i = 0; i < noBndNodes; i++) b->Set(bndNodes[i],value);

  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;

}
//-----------------------------------------------------------------------------
/*
int outflowBC( SparseMatrix* A, Vector* b, MV_ColMat<real>& bndLim, Field& u1, real Re, Grid* grid, int nsys, int eq, 
	       MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  MV_Vector<int> bNodes( grid->GetNoNodes() );
  int noBndNodes;
  double *diriVal;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);
  
  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }
 
  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCVal.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  A->SetRowIdentity(bndNodes,noBndNodes);

  Node *n;
  for( i = 0; i < noBndNodes; i++){
    n = grid->GetNode(bNodes(i));
    b->Set(bndNodes[i],0.0);
    for (j=0;j<n->GetNoCellNeighbors();j++) b->Add(bndNodes[i],(1.0/Re)*u1.EvalGradient(n->GetCellNeighbor(j),0));
    b->Mult(bndNodes[i],real(n->GetNoCellNeighbors()));
  }

  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;
}
//-----------------------------------------------------------------------------
int outflowBC( Vector* b, MV_ColMat<real>& bndLim, Field& u1, real Re, Grid* grid, int nsys, int eq, 
	       MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  MV_Vector<int> bNodes( grid->GetNoNodes() );
  int noBndNodes;
  double *diriVal;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);
  
  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }
 
  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCVal.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  Node *n;
  for( i = 0; i < noBndNodes; i++){
    n = grid->GetNode(bNodes(i));
    b->Set(bndNodes[i],0.0);
    for (j=0;j<n->GetNoCellNeighbors();j++) b->Add(bndNodes[i],(1.0/Re)*u1.EvalGradient(n->GetCellNeighbor(j),0));
    b->Mult(bndNodes[i],real(n->GetNoCellNeighbors()));
  }

  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;
}
*/
//-----------------------------------------------------------------------------
int getPerList( MV_ColMat<real>& bndLim1, MV_ColMat<real>& bndLim2, Grid* g, MV_Vector<int>& perList )
{
  // creates a nodal list where the entries for the slave nodes (bndLim2) are changed 
  // to the number of the corresponding master nodes (bndLim1), else same number as entry  
  int ierr;
  int i,j,k;
  MV_Vector<int> bNodes1;
  MV_Vector<int> bNodes2;
  int noBndNodes1;
  int noBndNodes2;
  //real *tmpvec, one = 1.0, minus = -1.0, zero = 0.0;

  // get node numbering
  getBndNodes(bNodes1, &noBndNodes1, bndLim1, g);
  getBndNodes(bNodes2, &noBndNodes2, bndLim2, g);

  // check compability of master-slave nodes
  if ( noBndNodes1 != noBndNodes2 ){
    cout << "ERROR: Periodic nodes not compatible! (not the same number of nodes)" << endl;
    exit(1);
  }

  // if the list is empty, initialize it, else add new master-slave info
  int noNodes = g->GetNoNodes();
  if ( perList.size() == 0 ){
    perList.newsize(noNodes);
    for( i = 0; i < noNodes; i++) perList(i) = i;
  }
  
  Point pi, pj;
  
  for( i = 0; i < noBndNodes1; i++){
    pi = g->GetNode(bNodes1(i))->GetCoord();
    for( j = 0; j < noBndNodes2; j++){
      pj = g->GetNode(bNodes2(j))->GetCoord();
      if( ( pi.x == pj.x && pi.y == pj.y ) || 
	  ( pi.x == pj.x && pi.z == pj.z ) || 
	  ( pi.y == pj.y && pi.z == pj.z ) ){
	//perList(bNodes1(i)) = bNodes1(j);
	perList(bNodes2(j)) = bNodes1(i);
      }
    }
  }
  
}
//-----------------------------------------------------------------------------
/*
int dirichletBCInflowGivenField( Vec b, MV_ColMat<real>& bndLim, Grid* grid, 
				 int nsys, int eq, real value, MV_ColMat<real>& field , 
				 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  Mat Atmp;
  nsys = -nsys;

  dirichletBCInflowGivenField( Atmp, b, bndLim, grid, nsys, eq, value, field, DCIdx, DCVal );
  
  return 0;

}
//-----------------------------------------------------------------------------
int dirichletBCInflowGivenField( Mat A, Vec b, MV_ColMat<real>& bndLim, Grid* grid, 
				 int nsys, int eq, real value, MV_ColMat<real>& field, 
				 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int MATRIX = 1;
  if ( nsys < 0 ){
    nsys = -nsys;
    MATRIX = 0;
  }

  int ierr;
  int i,j;
  int *bndNodes; 
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);
  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  IS bndRows;
  ierr = ISCreateGeneral(PETSC_COMM_SELF, noBndNodes, bndNodes, &bndRows);
  if ( MATRIX == 1 ){
    ierr = MatZeroRows( A, bndRows, &one); CHKERRQ(ierr);
  }
  
  ierr = VecGetArray( b, &tmpvec);
  real x,y,z;
  real pi = 3.14159;
  real k = 10.0;
  j = 0;
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = field(bNodes(i),eq);
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );
 
  ierr = PetscFree( bndNodes );
  ierr = ISDestroy( bndRows );

  PetscFunctionReturn(0);
}
//-----------------------------------------------------------------------------
int dirichletBC_GRID( Mat A, Vec b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int *bndNodes; 
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  real k  = 2.0;
  real pi = 3.14159;
  real x,y,z;

  int cnt = 0;
  
  real x0,y0_1,z0_1,x1,y1_1,z1_1;
  real y0_2,z0_2,y1_2,z1_2;
  real y0_3,z0_3,y1_3,z1_3;
  real y0_4,z0_4,y1_4,z1_4;

  MV_Vector<int> tmp( grid->GetNoNodes() );

  x0 = 0.5;
  x1 = 0.625;

  y0_1 = 0.125;
  y1_1 = 0.375;
  z0_1 = 0.125;
  z1_1 = 0.375;

  y0_2 = 0.125;
  y1_2 = 0.375;
  z0_2 = 0.625;
  z1_2 = 0.875;

  y0_3 = 0.625;
  y1_3 = 0.875;
  z0_3 = 0.125;
  z1_3 = 0.375;

  y0_4 = 0.625;
  y1_4 = 0.875;
  z0_4 = 0.625;
  z1_4 = 0.875;

  Node *n;
  for ( i=0; i < grid->GetNoNodes(); i++ ){
	 n = grid->GetNode(i);
    if ( (n->GetCoord(0)>=x0) && (n->GetCoord(0)<=x1) ){ 
      if (    ((n->GetCoord(1)>y0_1)&&(n->GetCoord(1)<y1_1)&&(n->GetCoord(2)>z0_1)&&(n->GetCoord(2)<z1_1))
	   || ((n->GetCoord(1)>y0_2)&&(n->GetCoord(1)<y1_2)&&(n->GetCoord(2)>z0_2)&&(n->GetCoord(2)<z1_2))
	   || ((n->GetCoord(1)>y0_3)&&(n->GetCoord(1)<y1_3)&&(n->GetCoord(2)>z0_3)&&(n->GetCoord(2)<z1_3))
	   || ((n->GetCoord(1)>y0_4)&&(n->GetCoord(1)<y1_4)&&(n->GetCoord(2)>z0_4)&&(n->GetCoord(2)<z1_4)) ){
      } else{
	tmp(cnt) = i;
	cnt++;
      }
    }
  }
  
  bNodes.newsize(cnt);
  for ( i=0; i < cnt; i++ ){
    bNodes(i) = tmp(i);
  }
  noBndNodes = cnt;

  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  IS bndRows;
  ierr = ISCreateGeneral(PETSC_COMM_SELF, noBndNodes, bndNodes, &bndRows);
  ierr = MatZeroRows( A, bndRows, &one); CHKERRQ(ierr);
  
  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = 0.0; 
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );
 
  ierr = PetscFree( bndNodes );
  ierr = ISDestroy( bndRows );

  PetscFunctionReturn(0);
}
//-----------------------------------------------------------------------------
int dirichletBC_GRID( Vec b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int *bndNodes; 
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  real k  = 2.0;
  real pi = 3.14159;
  real x,y,z;
  
  int cnt = 0;

  real x0,y0_1,z0_1,x1,y1_1,z1_1;
  real y0_2,z0_2,y1_2,z1_2;
  real y0_3,z0_3,y1_3,z1_3;
  real y0_4,z0_4,y1_4,z1_4;

  MV_Vector<int> tmp( grid->GetNoNodes() );

  x0 = 0.5;
  x1 = 0.625;

  y0_1 = 0.125;
  y1_1 = 0.375;
  z0_1 = 0.125;
  z1_1 = 0.375;

  y0_2 = 0.125;
  y1_2 = 0.375;
  z0_2 = 0.625;
  z1_2 = 0.875;

  y0_3 = 0.625;
  y1_3 = 0.875;
  z0_3 = 0.125;
  z1_3 = 0.375;

  y0_4 = 0.625;
  y1_4 = 0.875;
  z0_4 = 0.625;
  z1_4 = 0.875;

  Node *n;
  for ( i=0; i < grid->GetNoNodes(); i++ ){
	 n = grid->GetNode(i);
    if ( (n->GetCoord(0)>=x0) && (n->GetCoord(0)<=x1) ){ 
      if (    ((n->GetCoord(1)>y0_1)&&(n->GetCoord(1)<y1_1)&&(n->GetCoord(2)>z0_1)&&(n->GetCoord(2)<z1_1))
	   || ((n->GetCoord(1)>y0_2)&&(n->GetCoord(1)<y1_2)&&(n->GetCoord(2)>z0_2)&&(n->GetCoord(2)<z1_2))
	   || ((n->GetCoord(1)>y0_3)&&(n->GetCoord(1)<y1_3)&&(n->GetCoord(2)>z0_3)&&(n->GetCoord(2)<z1_3))
	   || ((n->GetCoord(1)>y0_4)&&(n->GetCoord(1)<y1_4)&&(n->GetCoord(2)>z0_4)&&(n->GetCoord(2)<z1_4)) ){
      } else{
	tmp(cnt) = i;
	cnt++;
      }
    }
  }
  
  bNodes.newsize(cnt);
  for ( i=0; i < cnt; i++ ){
    bNodes(i) = tmp(i);
  }
  noBndNodes = cnt;

  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = 0.0; 
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );
 
  ierr = PetscFree( bndNodes );

  PetscFunctionReturn(0);
}
*/
//-----------------------------------------------------------------------------
int dirichletBC_GRID( SparseMatrix* A, Vector* b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;

  real k  = 2.0;
  real pi = 3.14159;
  real x,y,z;

  int cnt = 0;
  
  real x0,y0_1,z0_1,x1,y1_1,z1_1;
  real y0_2,z0_2,y1_2,z1_2;
  real y0_3,z0_3,y1_3,z1_3;
  real y0_4,z0_4,y1_4,z1_4;

  MV_Vector<int> tmp( grid->GetNoNodes() );

  x0 = 0.5;
  x1 = 0.625;

  y0_1 = 0.125;
  y1_1 = 0.375;
  z0_1 = 0.125;
  z1_1 = 0.375;

  y0_2 = 0.125;
  y1_2 = 0.375;
  z0_2 = 0.625;
  z1_2 = 0.875;

  y0_3 = 0.625;
  y1_3 = 0.875;
  z0_3 = 0.125;
  z1_3 = 0.375;

  y0_4 = 0.625;
  y1_4 = 0.875;
  z0_4 = 0.625;
  z1_4 = 0.875;

  Node *n;
  for ( i=0; i < grid->GetNoNodes(); i++ ){
    n = grid->GetNode(i);
    if ( (n->GetCoord(0)>=x0) && (n->GetCoord(0)<=x1) ){ 
      if (    ((n->GetCoord(1)>y0_1)&&(n->GetCoord(1)<y1_1)&&(n->GetCoord(2)>z0_1)&&(n->GetCoord(2)<z1_1))
	   || ((n->GetCoord(1)>y0_2)&&(n->GetCoord(1)<y1_2)&&(n->GetCoord(2)>z0_2)&&(n->GetCoord(2)<z1_2))
	   || ((n->GetCoord(1)>y0_3)&&(n->GetCoord(1)<y1_3)&&(n->GetCoord(2)>z0_3)&&(n->GetCoord(2)<z1_3))
	      || ((n->GetCoord(1)>y0_4)&&(n->GetCoord(1)<y1_4)&&(n->GetCoord(2)>z0_4)&&(n->GetCoord(2)<z1_4)) ){
      } else{
	tmp(cnt) = i;
	cnt++;
      }
    }
  }
  
  bNodes.newsize(cnt);
  for ( i=0; i < cnt; i++ ){
    bNodes(i) = tmp(i);
  }
  noBndNodes = cnt;

  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  A->SetRowIdentity(bndNodes,noBndNodes);
  
  for( i = 0; i < noBndNodes; i++) b->Set(bndNodes[i],0.0); 

  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;
}
//-----------------------------------------------------------------------------
int dirichletBC_GRID( Vector* b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int ierr;
  int i,j;
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;

  real k  = 2.0;
  real pi = 3.14159;
  real x,y,z;

  int cnt = 0;
  
  real x0,y0_1,z0_1,x1,y1_1,z1_1;
  real y0_2,z0_2,y1_2,z1_2;
  real y0_3,z0_3,y1_3,z1_3;
  real y0_4,z0_4,y1_4,z1_4;

  MV_Vector<int> tmp( grid->GetNoNodes() );

  x0 = 0.5;
  x1 = 0.625;

  y0_1 = 0.125;
  y1_1 = 0.375;
  z0_1 = 0.125;
  z1_1 = 0.375;

  y0_2 = 0.125;
  y1_2 = 0.375;
  z0_2 = 0.625;
  z1_2 = 0.875;

  y0_3 = 0.625;
  y1_3 = 0.875;
  z0_3 = 0.125;
  z1_3 = 0.375;

  y0_4 = 0.625;
  y1_4 = 0.875;
  z0_4 = 0.625;
  z1_4 = 0.875;

  Node *n;
  for ( i=0; i < grid->GetNoNodes(); i++ ){
    n = grid->GetNode(i);
    if ( (n->GetCoord(0)>=x0) && (n->GetCoord(0)<=x1) ){ 
      if (    ((n->GetCoord(1)>y0_1)&&(n->GetCoord(1)<y1_1)&&(n->GetCoord(2)>z0_1)&&(n->GetCoord(2)<z1_1))
	   || ((n->GetCoord(1)>y0_2)&&(n->GetCoord(1)<y1_2)&&(n->GetCoord(2)>z0_2)&&(n->GetCoord(2)<z1_2))
	   || ((n->GetCoord(1)>y0_3)&&(n->GetCoord(1)<y1_3)&&(n->GetCoord(2)>z0_3)&&(n->GetCoord(2)<z1_3))
	      || ((n->GetCoord(1)>y0_4)&&(n->GetCoord(1)<y1_4)&&(n->GetCoord(2)>z0_4)&&(n->GetCoord(2)<z1_4)) ){
      } else{
	tmp(cnt) = i;
	cnt++;
      }
    }
  }
  
  bNodes.newsize(cnt);
  for ( i=0; i < cnt; i++ ){
    bNodes(i) = tmp(i);
  }
  noBndNodes = cnt;

  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  for( i = 0; i < noBndNodes; i++) b->Set(bndNodes[i],0.0); 

  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;
}
//-----------------------------------------------------------------------------
int dirichletBCInflow( SparseMatrix *A, Vector *b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int MATRIX = 1;
  if ( nsys < 0 ){
    nsys = -nsys;
    MATRIX = 0;
  }

  int ierr;
  int i,j;
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);

  int *bndNodes; 
  bndNodes = new int[noBndNodes]; 

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  if ( MATRIX == 1 ){
    A->SetRowIdentity(bndNodes,noBndNodes);
  }
  
  real x,y,z;
  real pi = 3.14159;
  real k = 10.0;

  bool bfs,bb,jet,les,turbulent_inflow;
  settings->Get("backward facing step",&bfs);
  settings->Get("bluff body",&bb);
  settings->Get("jet",&jet);
  settings->Get("LES benchmark",&les);

  settings->Get("turbulent inflow",&turbulent_inflow);
  
  j = 0;
  Point p;
  for( i = 0; i < noBndNodes; i++){
    p = grid->GetNode(bNodes(i))->GetCoord();
	 
    x = real( p.x );
    y = real( p.y );
    z = real( p.z );
	 
    if ( eq==0 ){
      if ( bfs ){
	if (y>=0.5) b->Set(bndNodes[i],64.0*(y-0.5)*(1.0-y)*z*(1.0-z));
      } else if ( bb ){
	b->Set(bndNodes[i],16.0*y*(1.0-y)*z*(1.0-z));
      } else if ( jet ){
	b->Set(bndNodes[i],16.0*y*(1.0-y)*z*(1.0-z));
      } else if ( les ){
	b->Set(bndNodes[i],2.0*y*(2.0-y));
      }	else{
	cout << "ERROR: Problem not implemented!" << endl;
	exit(1);
      }
    } else{
      b->Set(bndNodes[i],0.0);
    }
    if ( turbulent_inflow ){
      b->Add(bndNodes[i],(*turbulence)(i,eq));
    }	 
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = b->Get(bndNodes[i-(DCIdx.size()-noBndNodes)]);
  }

  delete bndNodes;	
}
//-----------------------------------------------------------------------------
int dirichletBCInflow( Vector* b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  SparseMatrix *Atmp;
  nsys = -nsys;

  dirichletBCInflow( Atmp, b, bndLim, grid, nsys, eq, value, turbulence, DCIdx, DCVal );
  
  return 0;
}
//-----------------------------------------------------------------------------
/*
int dirichletBCInflow( Mat A, Vec b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  int MATRIX = 1;
  if ( nsys < 0 ){
    nsys = -nsys;
    MATRIX = 0;
  }

  int ierr;
  int i,j;
  int *bndNodes; 
  int noNodes = grid->GetNoNodes();
  MV_Vector<int> bNodes(noNodes);
  int noBndNodes;
  double *diriVal;
  Scalar *tmpvec, one = 1.0;

  PetscFunctionBegin;

  ierr = getBndNodes(bNodes, &noBndNodes, bndLim, grid);
  PetscMalloc(noBndNodes*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(noBndNodes*sizeof(int));

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] = bNodes(i) * nsys;
    bndNodes[i] += eq;
  }

  // Set new size for Dirichlet vectors 
  MV_Vector<int> tmpVecIdx(DCIdx.size());
  MV_Vector<real> tmpVecVal(DCIdx.size());
  if ( DCIdx.size() != 0 ){
    for( i = 0; i < DCIdx.size(); i++){
      tmpVecIdx(i) = DCIdx(i);
      tmpVecVal(i) = DCVal(i);
    }
  }
  DCIdx.newsize(DCIdx.size()+noBndNodes);
  DCVal.newsize(DCVal.size()+noBndNodes);
  if ( DCIdx.size() != noBndNodes ){
    for( i = 0; i < DCIdx.size()-noBndNodes; i++){
      DCIdx(i) = tmpVecIdx(i);
      DCVal(i) = tmpVecVal(i);
    }
  }

  IS bndRows;
  ierr = ISCreateGeneral(PETSC_COMM_SELF, noBndNodes, bndNodes, &bndRows);
  if ( MATRIX == 1 ){
    ierr = MatZeroRows( A, bndRows, &one); CHKERRQ(ierr);
  }
  
  ierr = VecGetArray( b, &tmpvec);
  real x,y,z;
  real pi = 3.14159;
  real k = 10.0;

  bool bfs,bb,jet,les,turbulent_inflow;
  settings->Get("backward facing step",&bfs);
  settings->Get("bluff body",&bb);
  settings->Get("jet",&jet);
  settings->Get("LES benchmark",&les);

  settings->Get("turbulent inflow",&turbulent_inflow);
  
  j = 0;
  Point p;
  for( i = 0; i < noBndNodes; i++){
	 p = grid->GetNode(bNodes(i))->GetCoord();
	 
    x = real( p.x );
    y = real( p.y );
    z = real( p.z );
	 
    if ( eq==0 ){
      if ( bfs ){
	if (y>=0.5) tmpvec[bndNodes[i]] = 64.0*(y-0.5)*(1.0-y)*z*(1.0-z);
      } else if ( bb ){
	tmpvec[bndNodes[i]] = 16.0*y*(1.0-y)*z*(1.0-z);
      } else if ( jet ){
	tmpvec[bndNodes[i]] = 16.0*y*(1.0-y)*z*(1.0-z);
      } else if ( les ){
	tmpvec[bndNodes[i]] = 2.0*y*(2.0-y);
      }	else{
	cout << "ERROR: Problem not implemented!" << endl;
	exit(1);
      }
    } else{
      tmpvec[bndNodes[i]] = 0.0;
    }
    if ( turbulent_inflow ){
      tmpvec[bndNodes[i]] += (*turbulence)(i,eq);
    }	 
  }
  for( i = DCIdx.size()-noBndNodes; i < DCIdx.size(); i++){
    DCIdx(i) = bndNodes[i-(DCIdx.size()-noBndNodes)];
    DCVal(i) = tmpvec[bndNodes[i-(DCIdx.size()-noBndNodes)]];
  }
  ierr = VecRestoreArray( b, &tmpvec );
 
  //cout << "Dirichlet inflow BC applied on " << noBndNodes << " nodes" << endl; 

  ierr = PetscFree( bndNodes );
  ierr = ISDestroy( bndRows );

  PetscFunctionReturn(0);
}
//-----------------------------------------------------------------------------
int dirichletBCInflow( Vec b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal )
{
  Mat Atmp;
  nsys = -nsys;

  dirichletBCInflow( Atmp, b, bndLim, grid, nsys, eq, value, turbulence, DCIdx, DCVal );
  
  return 0;
}
//-----------------------------------------------------------------------------
int dirichletBCnoGeom2D( Mat A, Vec b, Grid* grid, int nsys, int eq)
{
  //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  //  This function is NOT general at all.
  //  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  int ierr;
  int i;
  int *bndNodes; 
  double *diriVal;
  Scalar *tmpvec, one = 1.0;
  IS bndRows;

  PetscFunctionBegin;

  PetscMalloc((grid->GetNoNodes()+1)*sizeof(int),&bndNodes);
  //bndNodes = (int *) PetscMalloc(grid->GetNoNodes*sizeof(int));

  int noBndNodes = 0;

  Node *n;
  for( i = 0; i < grid->GetNoNodes(); i++){
	 n = grid->GetNode(i);
    float x = n->GetCoord(0);
    float y = n->GetCoord(1);
    //if( x < 0.001 ){
    if( (x < 0.001) || (y < 0.001) ){
      //if( (x < 0.001) || (y < 0.001) || ((1.0-x) < 0.001) || ((1.0-y) < 0.001)){
      bndNodes[noBndNodes++] = i;
      cout <<"["<<i<<"]  "<< x << "   " << y << endl;
    }
  }

  for( i = 0; i < noBndNodes; i++){
    bndNodes[i] *= nsys;
    bndNodes[i] += eq;
  }

  ierr = ISCreateGeneral(PETSC_COMM_SELF, noBndNodes, bndNodes, &bndRows);
  ierr = MatZeroRows( A, bndRows, &one); CHKERRQ(ierr);

  ierr = VecGetArray( b, &tmpvec);
  for( i = 0; i < noBndNodes; i++){
    tmpvec[bndNodes[i]] = 0.0;
  }
  ierr = VecRestoreArray( b, &tmpvec );
  
  ierr = PetscFree( bndNodes );
  ierr = ISDestroy( bndRows );
  PetscFunctionReturn(0);
}
*/
//-----------------------------------------------------------------------------
int writeInpTD(Grid* grid, MV_ColMat<real> umat, int nsys, int noTimeSteps, int dt)
{
  FILE *fp;
  int ierr;
  int i,j,k;
  int el;
  real *solArr;
  int NODE_DATA = 1;

  int noNodes = grid->GetNoNodes();
  int noElms = grid->GetNoCells();

  fp = fopen("u_timedep.inp","w");

  fprintf(fp,"%i\n",noTimeSteps);
  fprintf(fp,"data\n");
  fprintf(fp,"step1 no.1, t = %f\n",dt);
  fprintf(fp, "%i %i %i 0 0\n",noNodes,noElms, nsys);

  Node *n;
  for( i = 0; i < noNodes; i++){
	 n = grid->GetNode(i);
    fprintf(fp, "%i %f %f %f\n", i+1,
	    n->GetCoord(0),
	    n->GetCoord(1),
	    n->GetCoord(2));
  }

  Cell *c;
  for( i = 0; i < noElms; i++){
	 c = grid->GetCell(i);
    fprintf(fp, "%i %i tet %i %i %i %i\n", i+1,
				c->GetMaterial(),
				c->GetNode(0)+1,
				c->GetNode(1)+1,
				c->GetNode(2)+1,
				c->GetNode(3)+1);
  }

  fprintf(fp,"%i ", nsys);
  for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
  fprintf(fp,"\n");
  for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);

  for(i = 0; i < noNodes; i++){
    fprintf(fp, "%i ", i+1); 
    for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", umat(noNodes+i,j));
    fprintf(fp, "\n"); 
  }

  for ( k=1; k < noTimeSteps; k++ ){

    fprintf(fp,"step%i no.1, t = %f\n",k,k*dt);
  
    fprintf(fp,"%i ", nsys);
    for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
    fprintf(fp,"\n");
    for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);
   
    for(i = 0; i < noNodes; i++){
      fprintf(fp, "%i ", i+1); 
      for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", umat(k*noNodes+i,j));
      fprintf(fp, "\n"); 
    }
  }
   
  fclose(fp);

  printf("Solution written to file: u_timedep.inp\n");

  return 0;
}
//-----------------------------------------------------------------------------
int writeMfile(char *file, Grid *grid, MV_ColMat<real> u, int nsys, int comp)
{
  FILE *fp;
  int ierr;
  int i, j;
  int el;
  real *solArr;

  fp = fopen(file,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
  for(i = 0; i < grid->GetNoNodes(); i++){
    for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", u(i,comp+j));
    fprintf(fp, "\n"); 
  }
  fclose(fp);

  printf("Solution written to file: %s\n", file);

  return 0;
}
//-----------------------------------------------------------------------------
int writeInp(char *file, Grid *grid, MV_Vector<real> u, int nsys )
{
  FILE *fp;
  int ierr;
  int i, j;
  int el;
  real *solArr;
  int NODE_DATA = 1;

  fp = fopen(file,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");

  fprintf(fp, "%i %i %i 0 0\n",grid->GetNoNodes(),grid->GetNoCells(), nsys);

  Node *n;
  for( i = 0; i < grid->GetNoNodes(); i++){
	 n = grid->GetNode(i);
    fprintf(fp, "%i %f %f %f\n", i+1,
	    n->GetCoord(0),
	    n->GetCoord(1),
	    n->GetCoord(2));
  }

  Cell *c;
  for( i = 0; i < grid->GetNoCells(); i++){
	 c = grid->GetCell(i);
    fprintf(fp, "%i %i tet %i %i %i %i\n", i+1,
				c->GetMaterial(),
				c->GetNode(0)+1,
				c->GetNode(1)+1,
				c->GetNode(2)+1,
				c->GetNode(3)+1);
  }

  if( NODE_DATA ){ 
    fprintf(fp,"%i ", nsys);
    for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
    fprintf(fp,"\n");
    for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);
    for(i = 0; i < grid->GetNoNodes(); i++){
      fprintf(fp, "%i ", i+1); 
      for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", u(i*nsys+j));
      fprintf(fp, "\n"); 
    }
  }
  else{
    fprintf(fp,"1 1");
    fprintf(fp,"\n");
    fprintf(fp,"u , Unit\n");
    for(i = 0; i < grid->GetNoCells(); i++){
		c = grid->GetCell(i);
      fprintf(fp, "%i ", i+1); 
      fprintf(fp, "%i \n", c->GetMaterial()); 
    }
  }
  fclose(fp);

  printf("Solution written to file: %s\n", file);

  return 0;
}
//-----------------------------------------------------------------------------
int readInp( char *file, Grid *grid, MV_Vector<real>& u, int nsys )
{
  FILE *fp;
  int ierr;
  int i,j;
  int el;
  real *solArr;
  int NODE_DATA = 1;

  int tmp_int1;
  int tmp_int2;
  int tmp_int3;
  int tmp_int4;
  int tmp_int5;
  int tmp_int6;
  int tmp_int;
  real tmp_real;
  real tmp_real1;
  real tmp_real2;
  real tmp_real3;
  char tmp_str1[10];

  fp = fopen(file,"r"); //if (!fp) SETERRA(1,1,"Cannot open grid file");

  fscanf(fp, "%i %i %i %i %i \n", &tmp_int1, &tmp_int2, &tmp_int3, &tmp_int4, &tmp_int5);
  for( i = 0; i < grid->GetNoNodes(); i++){
    fscanf(fp, "%i %lf %lf %lf\n", &tmp_int1, &tmp_real1, &tmp_real2, &tmp_real3);
  }
  for( i = 0; i < grid->GetNoCells(); i++){
    fscanf(fp, "%i %i %s %i %i %i %i\n", &tmp_int1, &tmp_int2, &tmp_str1, &tmp_int3, &tmp_int4, &tmp_int5, &tmp_int6);
  }
  fscanf(fp,"%i ", &tmp_int);
  for ( i = 0; i < nsys; i++){
    fscanf(fp,"%i", &tmp_int);
  }
  fscanf(fp,"\n");
  for ( i = 0; i < nsys; i++){
    fscanf(fp,"u %i, Unit\n", &tmp_int );
  }
  for(i = 0; i < grid->GetNoNodes(); i++){
    fscanf(fp, "%i ", &tmp_int);
    for ( j=0; j < nsys; j++ ){
      fscanf(fp, "%lf ", &tmp_real);
      u(i*nsys+j) = tmp_real;
    }
    fscanf(fp, "\n"); 
  }
  fclose(fp);
  
  printf("Read from file: %s\n", file);

  return 0;
}
//-----------------------------------------------------------------------------
int writeInp(char *file, Grid *grid, MV_ColMat<real> u, int nsys, int firstcomp)
{
  FILE *fp;
  int ierr;
  int i, j;
  int el;
  real *solArr;
  int NODE_DATA = 1;

  fp = fopen(file,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");

  fprintf(fp, "%i %i %i 0 0\n",grid->GetNoNodes(),grid->GetNoCells(), nsys);

  Node *n;
  for( i = 0; i < grid->GetNoNodes(); i++){
	 n = grid->GetNode(i);
    fprintf(fp, "%i %f %f %f\n", i+1,
	    n->GetCoord(0),
	    n->GetCoord(1),
	    n->GetCoord(2));
  }

  Cell *c;
  for( i = 0; i < grid->GetNoCells(); i++){
	 c = grid->GetCell(i);
    fprintf(fp, "%i %i tet %i %i %i %i\n", i+1,
				c->GetMaterial(),
				c->GetNode(0)+1,
				c->GetNode(1)+1,
				c->GetNode(2)+1,
				c->GetNode(3)+1);
  }

  if( NODE_DATA ){ 
    fprintf(fp,"%i ", nsys);
    for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
    fprintf(fp,"\n");
    for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);
    for(i = 0; i < grid->GetNoNodes(); i++){
      fprintf(fp, "%i ", i+1); 
      for ( j = 0; j < nsys; j++){
	fprintf(fp, "%f ", u(i,firstcomp+j));
      }
      fprintf(fp, "\n"); 
    }
  }else{
    fprintf(fp,"1 1");
    fprintf(fp,"\n");
    fprintf(fp,"u , Unit\n");
    for(i = 0; i < grid->GetNoCells(); i++){
		c = grid->GetCell(i);
      fprintf(fp, "%i ", i+1); 
      fprintf(fp, "%i \n", c->GetMaterial());
    }
  }
  fclose(fp);

  printf("Solution written to file: %s\n", file);

  return 0;
}
//-----------------------------------------------------------------------------
int readInp(char *file, Grid *grid, MV_ColMat<real>& u, int nsys, int firstcomp)
{
  FILE *fp;
  int ierr;
  int i,j;
  int el;
  real *solArr;
  int NODE_DATA = 1;

  int tmp_int1;
  int tmp_int2;
  int tmp_int3;
  int tmp_int4;
  int tmp_int5;
  int tmp_int6;
  int tmp_int;
  real tmp_real;
  real tmp_real1;
  real tmp_real2;
  real tmp_real3;
  char tmp_str1[10];

  fp = fopen(file,"r"); //if (!fp) SETERRA(1,1,"Cannot open grid file");

  fscanf(fp, "%i %i %i %i %i \n", &tmp_int1, &tmp_int2, &tmp_int3, &tmp_int4, &tmp_int5);
  for( i = 0; i < grid->GetNoNodes(); i++){
    fscanf(fp, "%i %lf %lf %lf\n", &tmp_int1, &tmp_real1, &tmp_real2, &tmp_real3);
  }
  for( i = 0; i < grid->GetNoCells(); i++){
    fscanf(fp, "%i %i %s %i %i %i %i\n", &tmp_int1, &tmp_int2, &tmp_str1, &tmp_int3, &tmp_int4, &tmp_int5, &tmp_int6);
  }
  fscanf(fp,"%i ", &tmp_int);
  for ( i = 0; i < nsys; i++){
    fscanf(fp,"%i", &tmp_int);
  }
  fscanf(fp,"\n");
  for ( i = 0; i < nsys; i++){
    fscanf(fp,"u %i, Unit\n", &tmp_int );
  }
  for(i = 0; i < grid->GetNoNodes(); i++){
    fscanf(fp, "%i ", &tmp_int);
    for ( j=0; j < nsys; j++ ){
      fscanf(fp, "%lf ", &tmp_real);
      u(i,firstcomp+j) = tmp_real;
    }
    fscanf(fp, "\n"); 
  }
  fclose(fp);
  
  printf("Read from file: %s\n", file);

  return 0;
}
//-----------------------------------------------------------------------------
/*
int writeInp(char *file, Grid *grid, Vec *u, int nsys)
{
  FILE *fp;
  int ierr;
  int i, j;
  int el;
  Scalar *solArr;
  int NODE_DATA = 1;

  fp = fopen(file,"w"); if (!fp) SETERRA(1,1,"Cannot open grid file");

  fprintf(fp, "%i %i %i 0 0\n",grid->GetNoNodes(),grid->GetNoCells(), nsys);

  for( i = 0; i < grid->GetNoNodes(); i++){
  n = grid->GetNode(i);
    fprintf(fp, "%i %f %f %f\n", i+1,
	    n->GetCoord(0),
	    n->GetCoord(1),
	    n->GetCoord(2));
  }
  for( i = 0; i < grid->GetNoCells(); i++){
    fprintf(fp, "%i %i tet %i %i %i %i\n", i+1,
	    grid->getMaterialType(i),
	    grid->loc2glob(i,0)+1,
	    grid->loc2glob(i,1)+1,
	    grid->loc2glob(i,2)+1,
	    grid->loc2glob(i,3)+1);
  }

  ierr = VecGetArray(*u, &solArr);

  if( NODE_DATA ){ 
    fprintf(fp,"%i ", nsys);
    for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
    fprintf(fp,"\n");
    for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);
    for(i = 0; i < grid->GetNoNodes(); i++){
      fprintf(fp, "%i ", i+1); 
      for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", solArr[j+i*nsys]);
      fprintf(fp, "\n"); 
    }
  }else{
    fprintf(fp,"1 1");
    fprintf(fp,"\n");
    fprintf(fp,"u , Unit\n");
    for(i = 0; i < grid->GetNoCells(); i++){
      fprintf(fp, "%i ", i+1); 
      fprintf(fp, "%i \n", grid->getMaterialType(i)); 
    }
  }
  fclose(fp);

  ierr = VecRestoreArray(*u, &solArr); CHKERRQ( ierr );

  printf("Solution written to file: %s\n", file);

  return 0;
}
*/
//-----------------------------------------------------------------------------
// int writeInp2D(char *file, Grid *grid, Vec *u, int nsys)
// {
//   FILE *fp;
//   int ierr;
//   int i, j;
//   int el;
//   Scalar *solArr;
//   int NODE_DATA = 1;

//   fp = fopen(file,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");

//   fprintf(fp, "%i %i %i 0 0\n",grid->GetNoNodes(),grid->GetNoCells(), nsys);

//   for( i = 0; i < grid->GetNoNodes(); i++){
//     fprintf(fp, "%i %f %f 0.00000\n", i+1,
// 	    n->GetCoord(0),
// 	    n->GetCoord(1));
//   }
//   for( i = 0; i < grid->GetNoCells(); i++){
//     ElementType etyp = grid->getElementType(i);
//     if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i quad %i %i %i %i\n", i+1,
// 	    grid->getMaterialType(i),
// 	    grid->loc2glob(i,0)+1,
// 	    grid->loc2glob(i,1)+1,
// 	    grid->loc2glob(i,2)+1,
// 	    grid->loc2glob(i,3)+1);
//     else //if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i tri %i %i %i\n", i+1,
// 	    grid->getMaterialType(i),
// 	    grid->loc2glob(i,0)+1,
// 	    grid->loc2glob(i,1)+1,
// 	    grid->loc2glob(i,2)+1);
//   }

//   ierr = VecGetArray(*u, &solArr);

//   if( NODE_DATA ){ 
//     fprintf(fp,"%i ", nsys);
//     for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
//     fprintf(fp,"\n");
//     for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);
//     for(i = 0; i < grid->GetNoNodes(); i++){
//       fprintf(fp, "%i ", i+1); 
//       for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", solArr[j+i*nsys]);
//       fprintf(fp, "\n"); 
//     }
//   }else{
//     fprintf(fp,"1 1");
//     fprintf(fp,"\n");
//     fprintf(fp,"u , Unit\n");
//     for(i = 0; i < grid->GetNoCells(); i++){
//       fprintf(fp, "%i ", i+1); 
//       fprintf(fp, "%i \n", grid->getMaterialType(i)); 
//     }
//   }
//   fclose(fp);

//   ierr = VecRestoreArray(*u, &solArr); CHKERRQ( ierr );

//   printf("Solution written to file: %s\n", file);

//   return 0;
// }
//-----------------------------------------------------------------------------
// int writeSol2D(char *file1, char *file2, char *file3, Grid *grid, Vec *u, int nsys)
// {
//   FILE *fp;
//   int ierr;
//   int i,j,k,l;
//   int el;
//   Scalar *solArr;
//   int NODE_DATA = 1;

//   fp = fopen(file1,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
//   for( i = 0; i < grid->GetNoNodes(); i++){
//     fprintf(fp, "%f %f\n",
// 	    n->GetCoord(0),
// 	    n->GetCoord(1));
//   }
//   fclose(fp);

//   fp = fopen(file2,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
//   for( j = 0; j < grid->GetNoCells(); j++){
//     ElementType etyp = grid->getElementType(j);
//     if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i %i %i\n", 
// 	    grid->loc2glob(j,0)+1,
// 	    grid->loc2glob(j,1)+1,
// 	    grid->loc2glob(j,2)+1,
// 	    grid->loc2glob(j,3)+1);
//     else //if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i %i\n", 
// 	    grid->loc2glob(j,0)+1,
// 	    grid->loc2glob(j,1)+1,
// 	    grid->loc2glob(j,2)+1);
//   }
//   fclose(fp);


//   fp = fopen(file3,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
//   ierr = VecGetArray(*u, &solArr);
 
//   for(k = 0; k < grid->GetNoNodes(); k++){
//     for ( l = 0; l < nsys; l++) fprintf(fp, "%f ", solArr[l+k*nsys]);
//     fprintf(fp, "\n"); 
//   }
//   fclose(fp);

//   ierr = VecRestoreArray(*u, &solArr); CHKERRQ( ierr );

//   printf("Solution written to file: %s\n", file3);

//   return 0;
// }
//-----------------------------------------------------------------------------
// int writeInp2Dcontour(char *file, Grid *grid, Vec *u, int nsys){

//   FILE *fp;
//   int ierr;
//   int i, j;
//   int el;
//   Scalar *solArr;
//   int NODE_DATA = 1;

//   fp = fopen(file,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");

//   fprintf(fp, "%i %i %i 0 0\n",grid->GetNoNodes(),grid->GetNoCells(), nsys);

//   ierr = VecGetArray(*u, &solArr);
//   int THREED=1;
//   if(THREED) {
//   for( i = 0; i < grid->GetNoNodes(); i++){
//     fprintf(fp, "%i %f %f %f\n", i+1,
//             n->GetCoord(0),
//             n->GetCoord(1),
//             solArr[i]);
//   }
//   }else {
//   for( i = 0; i < grid->GetNoNodes(); i++){
//     fprintf(fp, "%i %f %f 0.00000\n", i+1,
//             n->GetCoord(0),
//             n->GetCoord(1));
//   }
//   }
//   for( i = 0; i < grid->GetNoCells(); i++){
//     ElementType etyp = grid->getElementType(i);
//     if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i quad %i %i %i %i\n", i+1,
//             grid->getMaterialType(i),
//             grid->loc2glob(i,0)+1,
//             grid->loc2glob(i,1)+1,
//             grid->loc2glob(i,2)+1,
//             grid->loc2glob(i,3)+1);
//     else //if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i tri %i %i %i\n", i+1,
//             grid->getMaterialType(i),
//             grid->loc2glob(i,0)+1,
//             grid->loc2glob(i,1)+1,
//             grid->loc2glob(i,2)+1);
//   }

//   if( NODE_DATA ){ 
//     fprintf(fp,"%i ", nsys);
//     for ( i = 0; i < nsys; i++) fprintf(fp,"1 ");
//     fprintf(fp,"\n");
//     for ( i = 0; i < nsys; i++) fprintf(fp,"u %i, Unit\n",i+1);
//     for(i = 0; i < grid->GetNoNodes(); i++){
//       fprintf(fp, "%i ", i+1); 
//       for ( j = 0; j < nsys; j++) fprintf(fp, "%f ", solArr[j+i*nsys]);
//       fprintf(fp, "\n"); 
//     }
//   }else{
//     fprintf(fp,"1 1");
//     fprintf(fp,"\n");
//     fprintf(fp,"u , Unit\n");
//     for(i = 0; i < grid->GetNoCells(); i++){
//       fprintf(fp, "%i ", i+1); 
//       fprintf(fp, "%i \n", grid->getMaterialType(i)); 
//     }
//   }
//   fclose(fp);

//   ierr = VecRestoreArray(*u, &solArr); CHKERRQ( ierr );

//   printf("Solution written to file: %s\n", file);

//   return 0;
// }
//-----------------------------------------------------------------------------
// int writeSol2Dsys(char *file1, char *file2, char *file3, char *file4, Grid *grid, Vec *u, int nsys)
// {
//   FILE *fp;
//   int ierr;
//   int i,j,k,l;
//   int el;
//   Scalar *solArr;
//   int NODE_DATA = 1;

//   fp = fopen(file1,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
//   for( i = 0; i < grid->GetNoNodes(); i++){
//     fprintf(fp, "%f %f\n",
// 	    n->GetCoord(0),
// 	    n->GetCoord(1));
//   }
//   fclose(fp);

//   fp = fopen(file2,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
//   for( j = 0; j < grid->GetNoCells(); j++){
//     ElementType etyp = grid->getElementType(j);
//     if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i %i %i\n", 
// 	    grid->loc2glob(j,0)+1,
// 	    grid->loc2glob(j,1)+1,
// 	    grid->loc2glob(j,2)+1,
// 	    grid->loc2glob(j,3)+1);
//     else //if(etyp==ELMQUAD1)
//     fprintf(fp, "%i %i %i\n", 
// 	    grid->loc2glob(j,0)+1,
// 	    grid->loc2glob(j,1)+1,
// 	    grid->loc2glob(j,2)+1);
//   }
//   fclose(fp);


//   fp = fopen(file3,"w"); //if (!fp) SETERRA(1,1,"Cannot open grid file");
//   ierr = VecGetArray(*u, &solArr);
 
//   for(k = 0; k < grid->GetNoNodes(); k++){
//     for ( l = 0; l < nsys; l++) fprintf(fp, "%f ", solArr[l+k*nsys]);
//     fprintf(fp, "\n"); 
//   }
//   fclose(fp);

//   ierr = VecRestoreArray(*u, &solArr); CHKERRQ( ierr );

//   printf("Solution written to file: %s\n", file3);

//   return 0;
// }
//-----------------------------------------------------------------------------


