#ifndef __UTIL_H
#define __UTIL_H

#include "Globals.hh"
#include <SparseMatrix.hh>
#include <Vector.hh>

extern Globals *globals;

int getBndNodes(MV_Vector<int> &bndNodes, int* noBndNodes, MV_ColMat<real>& bndLim, Grid* grid);

/// Get perList containing master-slave node info 
int getPerList( MV_ColMat<real>& bndLim1, MV_ColMat<real>& bndLim2, Grid* g, MV_Vector<int>& perList );



/// Dirichlet b.c. for jet problem
int dirichletBC_GRID( SparseMatrix* A, Vector* b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );
int dirichletBC_GRID( Vector* b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );


/// Dirichlet b.c. for nodes within bndLim, matrix A and vector b  
int dirichletBC(SparseMatrix* A, Vector* b, MV_ColMat<real>& bndLim,
					 real value, Grid* grid, int nsys, int eq,
					 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );
  
/// Dirichlet b.c. for nodes within bndLim, vector b  
int dirichletBC( Vector* b, MV_ColMat<real>& bndLim,
					  real value, Grid* grid, int nsys, int eq, 
					  MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Outflow b.c. for nodes within bndLim, matrix A and vector b  
//int outflowBC( SparseMatrix* A, Vector* b, MV_ColMat<real>& bndLim, Field& u1, real Re, Grid* grid, int nsys, int eq, 
//	       MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Outflow b.c. for nodes within bndLim, vector b  
//int outflowBC( Vector* b, MV_ColMat<real>& bndLim, Field& u1, real Re, Grid* grid, int nsys, int eq, 
//	       MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Inflow dirichlet b.c., matrix A and vector b 
int dirichletBCInflow( SparseMatrix* A, Vector* b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Inflow dirichlet b.c., vector b 
int dirichletBCInflow( Vector* b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/*
/// Dirichlet b.c. for jet problem
int dirichletBC_GRID( Mat A, Vec b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );
int dirichletBC_GRID( Vec b, Grid* grid, int nsys, int eq, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Dirichlet b.c. for nodeList, matrix A and vector b  
int dirichletBC( Mat A, Vec b, MV_Vector<int>& nodeList, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );
  
/// Dirichlet b.c. for nodeList, vector b  
int dirichletBC( Vec b, MV_Vector<int>& nodeList, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Dirichlet b.c. for nodes within bndLim, matrix A and vector b  
int dirichletBC( Mat A, Vec b, MV_ColMat<real>& bndLim, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );
  
/// Dirichlet b.c. for nodes within bndLim, vector b  
int dirichletBC( Vec b, MV_ColMat<real>& bndLim, Scalar value, Grid* grid, int nsys, int eq, 
		 MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Outflow b.c. for nodes within bndLim, matrix A and vector b  
int outflowBC( Mat A, Vec b, MV_ColMat<real>& bndLim, Field& u1, real Re, Grid* grid, int nsys, int eq, 
	       MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Outflow b.c. for nodes within bndLim, vector b  
int outflowBC( Vec b, MV_ColMat<real>& bndLim, Field& u1, real Re, Grid* grid, int nsys, int eq, 
	       MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Inflow dirichlet b.c., matrix A and vector b 
int dirichletBCInflow( Mat A, Vec b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );

/// Inflow dirichlet b.c., vector b 
int dirichletBCInflow( Vec b, MV_ColMat<real>& bndLim, Grid* grid, int nsys, int eq, 
		       real value, MV_ColMat<real>* turbulence, MV_Vector<int>& DCIdx, MV_Vector<real>& DCVal );
*/

/// Write inp-file time dependent
int writeInpTD(Grid* grid, MV_ColMat<real> umat, int nsys, int noTimeSteps, int dt);

/// Read inp-file
int readInp(char *file, Grid *grid, MV_ColMat<real>& u, int nsys, int firstcomp);

/// Read inp-file
int readInp(char *file, Grid *grid, MV_Vector<real>& u, int nsys);

/// Write m-file
int writeMfile(char *file, Grid *grid, MV_ColMat<real> u, int nsys, int comp);

/// Write inp-file
int writeInp(char *file, Grid *grid, MV_ColMat<real> u, int nsys, int firstcomp);

/// Write inp-file
int writeInp(char *file, Grid *grid, MV_Vector<real> u, int nsys);

/// Write inp-file 2d
//int writeInp2D(char *file, Grid *grid, Vec *u, int nsys);

/// Write solution inp-file 2d
//int writeSol2D(char *file1, char *file2, char *file3, Grid *grid, Vec *u, int nsys);

/// Write solution inp-file 2d contour
//int writeInp2Dcontour(char *file, Grid *grid, Vec *u, int nsys);

//int dirichletBC( Mat A, Vec b,  int surfId, Scalar value, Grid* grid, int nsys, int eq);

/// Write solution inp-file 2d no geometry
//int dirichletBCnoGeom2D( Mat A, Vec b, Grid* grid, int nsys, int eq);

#endif
