#include <kraftwerk.h>

kw_bc my_bc(real x, real y, real z, int component);

int main(int argc, char **argv)
{
  // Set problem
  kw_set_problem("navier-stokes");
  
  // Set parameters
  kw_set_parameter("problem description", "Navier-Stokes equations in 3D");
  kw_set_parameter("grid file",           "../../../../data/grids/tetgrid_4_4_4.inp");
  kw_set_parameter("output file prefix",  "navier_stokes");

  kw_set_parameter("start time", 0.0);
  kw_set_parameter("final time", 1.0);

  kw_set_parameter("debug level",  0);

  // Set boundary conditions
  kw_set_boundary_conditions(my_bc);
  
  // Initialize kraftwerk
  kw_init(argc,argv);

  // Save parameters
  kw_save_parameters();

  // Solve the problem
  kw_solve();
  
  // Finished
  kw_end();
  
  return 0;
}

kw_bc my_bc(real x, real y, real z, int component)
{
  kw_bc bc;
  
  if ( (x == 0.0) && (component == 0) ){
	 bc.type = dirichlet;
	 bc.val  = 0.0;
  }
  else if ( (x == 0.0) && (component != 0) ){
	 bc.type = dirichlet;
	 bc.val  = 0.0;
  }
  else if ( y == 0.0 ){
	 bc.type = dirichlet;
	 bc.val  = 0.0;
  }
  else if ( y == 1.0 ){
	 bc.type = dirichlet;
	 bc.val  = 0.0;
  }
  else if ( z == 0.0 ){
	 bc.type = dirichlet;
	 bc.val  = 0.0;
  }
  else if ( z == 1.0 ){
	 bc.type = dirichlet;
	 bc.val  = 0.0;
  }

  return bc;
}
